from . import fontcell_main as fm
import os
import shutil


def run(config_path):
    print('version: 1.0.9')
    root_dir = os.path.dirname(os.path.realpath(__file__))
    os.mkdir(root_dir + os.sep + 'results')
    conf = str(config_path.split('file:')[1])
    fm.fontcell(conf)
    return


def run_demo():
    print('version: 1.0.9')
    root_dir = os.path.dirname(os.path.realpath(__file__))
    project_path = root_dir + os.sep + 'fontcell_demo'
    os.mkdir(project_path + os.sep + 'results')
    fm.fontcell(project_path + os.sep + 'demo_config.txt')
    print('output data at: ' + project_path + os.sep + 'results')
    return


def del_old_files():
    root_dir = os.path.dirname(os.path.realpath(__file__))
    project_path = root_dir + os.sep + 'fontcell_files'
    try:
        shutil.rmtree(project_path)
        print('old files deleted')
    except:
        print('There is no internal files yet')
    return


# Aim: This file will work as a main for a 'Ontology fusion'


# ##################### #
# Libraries and Modules #
# ##################### #

import numpy as np
from . import Tree_com as treeComp
from . import doc_managing as doc
from mpi4py import MPI
import bigmpi4py as tbx
from collections import OrderedDict
from . import OBO_Formating as obo
from . import coord_taker as coord
from . import circ_ontology as co
import os
from . import pyhtml as h
from . import makeFigures as make
import networkx as nx
# ######### #


def gen_graph_similarity_array(node1, ontology1, ontology2, subgraphs2, window_size, topological=True,
                               constriction_threshold=0.0, sequential_dictionary=None, topological_cosine=False,
                               topological_pearson=False, topological_euclidean=False):
    nodes2_i = np.array(ontology2.nodes)
    nodes2 = list(map(lambda a: eval(a), nodes2_i))

    subgraph1 = treeComp.subGraphGenerator(ontology1, node1, window_size)

    if topological:
        topological_matrixx = np.zeros((subgraphs2.__len__()), dtype=object)
        if not topological_cosine and not topological_euclidean and not topological_pearson:
            for i in range(len(subgraphs2)):
                topological_matrixx[i] = treeComp.g_similarity(subgraph1, subgraphs2[i], convergence_threshold=0.001,
                                                               obtain_loops_number=False)
        else:
            for j in range(len(subgraphs2)):
                topological_matrixx[j] = treeComp.g_similarity_noniterative(subgraph1, subgraphs2[j],
                                                                            topological_cosine, topological_euclidean,
                                                                            topological_pearson)
        return topological_matrixx
    else:
        match, value = treeComp.g_constriction_similarity(subgraph1, subgraphs2, node1, nodes2, sequential_dictionary,
                                                          window_size, constriction_threshold)
        return match, value


def gen_text_sequential_matrix(graph1, graph2, text_process=False, synonyms=False, word_list_to_clear1=None,
                               word_list_to_clear2=None, split_from1=None, split_since1=None, split_from2=None,
                               split_since2=None):
    nodes1 = list(graph1.nodes)
    nodes2 = list(graph2.nodes)

    nodes1_p = list(map(lambda a: a, list(map(lambda a: eval(a), nodes1))))
    nodes2_p = list(map(lambda a: a, list(map(lambda a: eval(a), nodes2))))

    if text_process:
        # print()
        # process the nodes_labeling
        if word_list_to_clear1 is not None:
            nodes1_t = treeComp.clearOntology_labels(nodes1_p, word_list_to_clear1, synonyms, split_from=split_from1,
                                                     split_since=split_since1)
        if word_list_to_clear2 is not None:
            nodes2_t = treeComp.clearOntology_labels(nodes2_p, word_list_to_clear2, synonyms, split_from=split_from2,
                                                     split_since=split_since2)
    else:
        nodes1_t = nodes1_p
        nodes2_t = nodes2_p

    if not synonyms:
        sequential_matrix = np.zeros([nodes1_t.__len__(), nodes2_t.__len__()], dtype=object)

        for i in range(nodes1_t.__len__()):
            for j in range(nodes2_t.__len__()):
                simil = treeComp.similarity(nodes1_t[i], nodes2_t[j])
                sequential_matrix[i][j] = simil

        return sequential_matrix

    elif synonyms:

        sequential_matrix = np.zeros([nodes1_t.__len__(), nodes2_t.__len__()], dtype=object)

        for i in range(nodes1_t.__len__()):
            # print(i)
            for j in range(nodes2_t.__len__()):
                simil = treeComp.similarity_synonyms(nodes1_t[i], nodes2_t[j])
                sequential_matrix[i][j] = simil
        return sequential_matrix


def fuse_graphs(graph1, graph2, assing_matrix, root):
    '''
    Fuse two graphs having a assignation matrix. V5: Another way to fuse... 'Translating fuse'
    '''
    # Make the dictionaries and introduce edges
    edges1 = list(graph1.edges)
    edges2 = list(graph2.edges)

    type_dictionary = {}
    assing_dictionary = {}
    assing_dictionary_inverse = {}
    typus_2_dictionary = {}

    for i in range(len(assing_matrix)):
        term1 = str(assing_matrix[i][0])
        term2 = str(assing_matrix[i][1])
        tipus = assing_matrix[i][4]
        both = [term1, term2]
        assing_dictionary.update({term2: term1})
        type_dictionary.update({tuple(both): tipus})
        assing_dictionary_inverse.update({term1: term2})
        typus_2_dictionary.update({term1: tipus})

    # Translate all the possibles graph2 nodes
    new_edges2 = np.zeros([len(edges2), 2], dtype=object)
    translated = []
    for i in range(len(edges2)):
        row = [edges2[i][0], edges2[i][1]]
        if edges2[i][0] in assing_dictionary:
            row[0] = assing_dictionary[edges2[i][0]]
            translated.append(row[0])
        if edges2[i][1] in assing_dictionary:
            row[1] = assing_dictionary[edges2[i][1]]
            translated.append(row[1])
        new_edges2[i] = row

    new_edges = list(map(lambda a: tuple(a), new_edges2))

    old_edges = list(map(lambda a: tuple(a), edges1))

    # fuse both list with no repeats
    if root is None:
        fusedlist = list(OrderedDict.fromkeys(old_edges + new_edges))
    else:
        fusedlist1 = list(OrderedDict.fromkeys(old_edges + new_edges))
        test_digraph = treeComp.buildDiGraph(fusedlist1)
        fused_edges = list(test_digraph.edges)
        fusedlist = []
        for i in range(len(fused_edges)):
            if test_digraph.has_node(fused_edges[i][0]) and test_digraph.has_node(fused_edges[i][1]):
                if nx.has_path(test_digraph, root, fused_edges[i][0]) and nx.has_path(test_digraph, root, fused_edges[i][1]):
                    fusedlist.append((fused_edges[i][0], fused_edges[i][1]))

    # add old/new for colouring
    finallist = np.zeros([len(fusedlist), 4], dtype=list)
    for i in range(len(fusedlist)):
        if fusedlist[i] not in edges1:
            finallist[i] = [fusedlist[i][0], fusedlist[i][1], 'new', 0]
            if fusedlist[i][0] in assing_dictionary_inverse:
                tipus = tuple([fusedlist[i][0], assing_dictionary_inverse[fusedlist[i][0]]])
                finallist[i][3] = type_dictionary[tipus]
        else:
            finallist[i] = [fusedlist[i][0], fusedlist[i][1], 'old', 0]
    return finallist, typus_2_dictionary


def equivalent_dictionaryV4(graph1, graph2, sequential, globalthreshold, localthreshold, subgraphs1, subgraphs2
                            , windowsize=4, simtopo=True, constriction_threshold=0.0, topological_cosine=False
                            , topological_pearson=False, topological_euclidean=False, topological_threshold=0.0):
    '''
    This function was a improvement of the another function with similar name.
    :param topological: matrix format (graph1.nodes x graph2.nodes)
    :param sequential: matrix format (graph1.nodes x graph2.nodes)
    :param globalthreshold: threshold of word similarity
    :param subgraphs1/subgraphs2: list of digraph object from every graph at the previous window size
    :param B_to_A: a analysis that search a topological candidate and now compares
    :return: matrix: graph1.__len__() x 5
    [node from graph1, node from graph2 assigned, topological score, sequence score, assignation type]
    '''
    # Pre-process
    graph1_nodes = list(map(lambda a: eval(a), graph1.nodes))
    graph2_nodes = list(map(lambda a: eval(a), graph2.nodes))

    # Explicit conversion string to float
    sequen_score = np.zeros(sequential.__len__(), dtype=object)

    for l in range(sequential.__len__()):
        sequen_score[l] = list(map(lambda a:float(a), sequential[l]))

    matchMatrix = np.zeros([graph1_nodes.__len__(), 5], dtype=object)
    # print('graph1_nodes.__len__():', graph1_nodes.__len__())
    # print('graph2_nodes.__len__():', graph2_nodes.__len__())
    # print('sequential: ', len(sequential), len(sequential[0]))
    dictionary_seq = dictionary_fromMatrix(graph1_nodes, graph2_nodes, sequential)
    # print('Topological: ', len(topological), len(topological[0]))

    # Sequential match
    sequential_founds_g1 = []
    sequential_founds_g2 = []

    seq_match= betters(graph1_nodes, graph2_nodes, dictionary_seq)
    count = 0
    for i in range(len(seq_match)):
        # if i == 88:
        #     print()
        if seq_match[i][2] >= globalthreshold:
            matchMatrix[i] = [seq_match[i][0],
                              seq_match[i][1],
                              seq_match[i][2],
                              'None',
                              'Sequential'
                              ]
            sequential_founds_g1.append(seq_match[i][0])
            sequential_founds_g2.append(seq_match[i][1])
        else:
            matchMatrix[i][0] = seq_match[i][0]
            count += 1
            continue

    # pre-generate all the sub-graphs in two arrays of objects in order to save resources
    # print('pregenering subgraphs')
    # subgraphs1 = np.zeros(len(graph1_nodes), dtype=object)
    # for i in range(len(graph1_nodes)):
    #     subgraphs1[i] = treeComp.subGraphGenerator(graph1, graph1_nodes[i], windowsize)  # a possible optimization here
    #     # print('graph1, subgraph: ', i)
    #
    # subgraphs2 = np.zeros(len(graph2_nodes), dtype=object)
    # for j in range(len(graph2_nodes)):
    #     subgraphs2[j] = treeComp.subGraphGenerator(graph2, graph2_nodes[j], windowsize)  # a possible optimization here
    #     # print('graph2, subgraph: ', j)
    #
    # # Topological match
    # subgraphs2 = list(map(lambda a: treeComp.subGraphGenerator(graph2, a, windowsize), graph2_nodes))
    if simtopo:
        for i in range(len(matchMatrix)):
            # print(i)
            if matchMatrix[i][1] == '0' or matchMatrix[i][1] == 0:
                # A non-sequential resolved node
                # Inside topological match, use the localSequence() tool, and add a mean threshold for accept or deny one of
                # assignations.
                # if a node wasn't found a match, pass to the next. (keep it blank)

                matrix = gen_graph_similarity_array(graph1_nodes[i], graph1, graph2, subgraphs2, windowsize
                                                    , topological_cosine=topological_cosine
                                                    , topological_pearson=topological_pearson
                                                    , topological_euclidean=topological_euclidean)
                # for n in range(len(matrix)):
                #     matrix[n] = float(matrix[n])
                event = False
                count = 0
                # Default not match assign
                matchMatrix[i] = [graph1_nodes[i],
                                  0,
                                  0,
                                  0,
                                  'Not Matched'
                                  ]
                while not event:
                     # print('node ' + str(i) + ', sub-loop: ' + str(count))
                    if count >= len(matrix):
                        # print('no')
                        event = True
                    pos = np.argmax(matrix)
                    candidate1 = graph2_nodes[pos]
                    if candidate1 not in seq_match and subgraphs1[i] != None and matrix[pos] >= topological_threshold:
                        # matrix = set_positionNegative(matrix, pos)
                        mean = localSequence(subgraphs1[i], subgraphs2[pos], dictionary_seq, node1=graph1_nodes[i], node2=candidate1)
                        if mean > localthreshold :
                            # print('yes')
                            matchMatrix[i] = [graph1_nodes[i],
                                              candidate1,
                                              dictionary_seq[(tuple(graph1_nodes[i]), tuple(candidate1))],
                                              mean,
                                              'Topological'
                                              ]

                            event = True
                            count += 1
                        else:
                            count += 1
                    else:
                        count += 1
                    matrix = set_positionNegative(matrix, pos)
            else:
                continue

    elif not simtopo:
        dictionary_of_sequentials =dictionary_from_betters(seq_match, globalthreshold)
        for i in range(len(matchMatrix)):
            # print(i)
            if matchMatrix[i][1] == '0' or matchMatrix[i][1] == 0:
                # A non-sequential resolved node
                match, value = gen_graph_similarity_array(graph1_nodes[i], graph1, graph2, subgraphs2, windowsize, topological=False,
                                                   constriction_threshold=constriction_threshold,
                                                   sequential_dictionary=dictionary_of_sequentials)
                # for n in range(len(matrix)):
                #     matrix[n] = float(matrix[n])

                # Default not match assign
                if match == None:
                    matchMatrix[i] = [graph1_nodes[i],
                                      0,
                                      0,
                                      0,
                                      'Not Matched'
                                      ]
                else:
                    # print('yes')
                    candidate = match
                    matchMatrix[i] = [graph1_nodes[i],
                                      candidate,
                                      dictionary_seq[(tuple(graph1_nodes[i]), tuple(candidate))],
                                      value,
                                      'Topological'
                                      ]
                continue

    return matchMatrix


def equivalent_dictionaryV4_parallelized(comm, graph1, graph2, sequential, globalthreshold, localthreshold, subgraphs1,
                                         subgraphs2 , windowsize=4, simtopo=True,constriction_threshold=0.0,
                                         topological_cosine=False, topological_pearson=False,
                                         topological_euclidean=False, topological_threshold=0.0):
    '''
    This function was a improvement of the another function with similar name.
    :param topological: matrix format (graph1.nodes x graph2.nodes)
    :param sequential: matrix format (graph1.nodes x graph2.nodes)
    :param globalthreshold: threshold of word similarity
    :param subgraphs1/subgraphs2: list of digraph object from every graph at the previous window size
    :param B_to_A: a analysis that search a topological candidate and now compares
    :return: matrix: graph1.__len__() x 5
    [node from graph1, node from graph2 assigned, topological score, sequence score, assignation type]
    '''
    # Pre-process
    graph1_nodes = list(map(lambda a: eval(a), graph1.nodes))
    graph2_nodes = list(map(lambda a: eval(a), graph2.nodes))

    # Explicit conversion string to float
    sequen_score = np.zeros(sequential.__len__(), dtype=object)

    for l in range(sequential.__len__()):
        sequen_score[l] = list(map(lambda a:float(a), sequential[l]))

    matchMatrix = np.zeros([graph1_nodes.__len__(), 5], dtype=object)
    # print('graph1_nodes.__len__():', graph1_nodes.__len__())
    # print('graph2_nodes.__len__():', graph2_nodes.__len__())
    # print('sequential: ', len(sequential), len(sequential[0]))
    dictionary_seq = dictionary_fromMatrix(graph1_nodes, graph2_nodes, sequential)
    # print('Topological: ', len(topological), len(topological[0]))

    # Sequential match
    sequential_founds_g1 = []
    sequential_founds_g2 = []

    seq_match= betters(graph1_nodes, graph2_nodes, dictionary_seq)
    count = 0
    for i in range(len(seq_match)):
        # if i == 88:
        #     print()
        if seq_match[i][2] >= globalthreshold:
            matchMatrix[i] = [seq_match[i][0],
                              seq_match[i][1],
                              seq_match[i][2],
                              'None',
                              'Sequential'
                              ]
            sequential_founds_g1.append(seq_match[i][0])
            sequential_founds_g2.append(seq_match[i][1])
        else:
            matchMatrix[i][0] = seq_match[i][0]
            count += 1
            continue

    matchMatrix1 = tbx.scatter(matchMatrix, comm)

    matchMatrix2 = equivalent_analysis_for_paralellized(matchMatrix1, simtopo, graph1_nodes, graph1, graph2, subgraphs2,
                                                        windowsize, graph2_nodes, seq_match, subgraphs1, dictionary_seq,
                                                        constriction_threshold, localthreshold, topological_cosine,
                                                        topological_pearson, topological_euclidean,
                                                        topological_threshold, globalthreshold)

    matchMatrix = tbx.gather(matchMatrix2, comm)

    return matchMatrix


def equivalent_analysis_for_paralellized(matchMatrix, simtopo, graph1_nodes, graph1, graph2, subgraphs2, windowsize,
                                         graph2_nodes, seq_match, subgraphs1, dictionary_seq, constriction_threshold,
                                         localthreshold, topological_cosine, topological_pearson, topological_euclidean,
                                         topological_threshold, globalthresgold):
    if simtopo:
        for i in range(len(matchMatrix)):
            # print(i)
            if matchMatrix[i][1] == '0' or matchMatrix[i][1] == 0:
                # A non-sequential resolved node
                # Inside topological match, use the localSequence() tool, and add a mean threshold for accept or deny one of
                # assignations.
                # if a node wasn't found a match, pass to the next. (keep it blank)

                matrix = gen_graph_similarity_array(graph1_nodes[i], graph1, graph2, subgraphs2, windowsize,
                                                    topological_cosine=topological_cosine,
                                                    topological_pearson= topological_pearson,
                                                    topological_euclidean=topological_euclidean)
                # for n in range(len(matrix)):
                #     matrix[n] = float(matrix[n])
                event = False
                count = 0
                # Default not match assign
                matchMatrix[i] = [graph1_nodes[i],
                                  0,
                                  0,
                                  0,
                                  'Not Matched'
                                  ]
                while not event:
                     # print('node ' + str(i) + ', sub-loop: ' + str(count))
                    if count >= len(matrix):
                        # print('no')
                        event = True
                    pos = np.argmax(matrix)
                    candidate1 = graph2_nodes[pos]
                    if candidate1 not in seq_match and subgraphs1[i] != None and matrix[pos] >= topological_threshold:
                        # matrix = set_positionNegative(matrix, pos)
                        mean = localSequence(subgraphs1[i], subgraphs2[pos], dictionary_seq, node1=graph1_nodes[i], node2=candidate1)
                        if mean > localthreshold :
                            # print('yes')
                            matchMatrix[i] = [graph1_nodes[i],
                                              candidate1,
                                              dictionary_seq[(tuple(graph1_nodes[i]), tuple(candidate1))],
                                              mean,
                                              'Topological'
                                              ]

                            event = True
                            count += 1
                        else:
                            count += 1
                    else:
                        count += 1
                    matrix = set_positionNegative(matrix, pos)
            else:
                continue

    elif not simtopo:
        dictionary_of_sequentials =dictionary_from_betters(seq_match, globalthresgold)
        for i in range(len(matchMatrix)):
            # print(i)
            if matchMatrix[i][1] == '0' or matchMatrix[i][1] == 0:
                # A non-sequential resolved node
                match, value = gen_graph_similarity_array(graph1_nodes[i], graph1, graph2, subgraphs2, windowsize, topological=False,
                                                   constriction_threshold=constriction_threshold,
                                                   sequential_dictionary=dictionary_of_sequentials)
                # for n in range(len(matrix)):
                #     matrix[n] = float(matrix[n])

                # Default not match assign
                if match == None:
                    matchMatrix[i] = [graph1_nodes[i],
                                      0,
                                      0,
                                      0,
                                      'Not Matched'
                                      ]
                else:
                    # print('yes')
                    candidate = match
                    matchMatrix[i] = [graph1_nodes[i],
                                      candidate,
                                      dictionary_seq[(tuple(graph1_nodes[i]), tuple(candidate))],
                                      value,
                                      'Topological'
                                      ]
                continue
    return matchMatrix


def dictionary_from_betters(sequential_matchs, globalthreshold):
    '''
    create a dictionary that you put node from ontology1 and give node from ontology2 assigned by betters.
    '''
    dictionary_sq = {}
    for i in range(len(sequential_matchs)):
        if sequential_matchs[i][2] >= globalthreshold:
            dictionary_sq.update({sequential_matchs[i][0]: sequential_matchs[i][1]})
    return dictionary_sq



def localSequence(subgraph1, subgraph2, dictioseq, node1=None, node2=None, inverted=False):
    '''
    Given two nodes, his graphs and the sequential matrix. Make the mean of the convolutional windows
    '''
    nodelist1 = list(subgraph1.nodes)
    nodelist2 = list(subgraph2.nodes)

    # take both node_problem
    if node1 != None:
        nodelist1.remove(node1)
    if node2 != None:
        nodelist2.remove(node2)

    bettervalues = np.zeros(nodelist1.__len__(), float)
    for i in range(nodelist1.__len__()):
        values = np.zeros(nodelist2.__len__(), dtype=float)
        for j in range(nodelist2.__len__()):
            if not inverted:
                values[j] = dictioseq[(tuple(nodelist1[i]), tuple(nodelist2[j]))]
            elif inverted:
                values[j] = dictioseq[(tuple(nodelist2[j]), tuple(nodelist1[i]))]
        bettervalues[i] = np.amax(values)

    mean = np.mean(bettervalues)
    return mean


def betters(nodes1, nodes2, dictio):
    '''
    Make a list of nodes from graph 1 to a list with nodes of graph 2 and his better value of assignation
    '''
    final_list = np.zeros(nodes1.__len__(), dtype=object)
    for i in range(len(nodes1)):
        # if i == 87:
        #     print(i)
        old_value = 0.0
        row = [0, 0, 0]
        for j in range(nodes2.__len__()):
            value = dictio[(tuple(nodes1[i]), tuple(nodes2[j]))]
            if value > old_value:
                old_value = value
                row = [nodes1[i], nodes2[j], value]
        final_list[i] = row
    return final_list


def dictionary_fromMatrix(graph1_nodes, graph2_nodes, sequential_matrix):
    # nodes1 = list(graph1.nodes)
    # nodes2 = list(graph2.nodes)
    # print('graph1_nodes.__len__():', graph1_nodes.__len__())
    # print('graph2_nodes.__len__():', graph2_nodes.__len__())
    # print('matrix: ', len(sequential_matrix), len(sequential_matrix[0]))
    dictionary = {}
    for i in range(len(graph1_nodes)):
        for j in range(len(graph2_nodes)):
            dictionary.update({(graph1_nodes[i], graph2_nodes[j]): float(sequential_matrix[i][j])})
    return dictionary


def set_positionNegative(array, position):
    '''
    Given an array change the symbol(positive or negative) for the value on position
    '''
    if isinstance(array[position], int):
        array[position] = -1 * array[position]
    elif isinstance(array[position], float):
        array[position] = -1.0 * array[position]
    '''
    for i in range(len(array)):
        if i == position and isinstance(array[i], int):
            array[i] = -1 * array[i]
        elif i == position and isinstance(array[i], float):
            array[i] = -1.0 * array[i]
    '''
    return array


def quality_controls(equivalent_matrix, fused_matrix, graph2, return_values=False):
    '''
    :param equivalent_matrix: Matrix of equivalence obtained during fusing ontology
    :param fused_matrix: final matrix from the ontology fusing process
    :return: Shows the results of quality control
    '''
    # Number of nodes and edges added
    nodes2 = list(graph2.nodes)
    original_edges = 0
    original_edges_list = []
    original_nodes = 0
    edges_add = 0
    all_nodes = []
    original_old_nodes = []
    nodes_add = 0
    all_edges = []
    onto2_nodes_added = []

    for i in range(len(fused_matrix)):
        all_edges.append(fused_matrix[i][0:2])
        if fused_matrix[i][2] == 'new':
            edges_add += 1
            if fused_matrix[i][0] not in all_nodes:
                nodes_add += 1
                all_nodes.append(fused_matrix[i][0])
            if fused_matrix[i][1] not in all_nodes:
                nodes_add +=1
                all_nodes.append(fused_matrix[i][1])
        else:
            original_edges += 1
            original_edges_list.append(fused_matrix[i][0:2])
            if fused_matrix[i][0] not in all_nodes:
                all_nodes.append(fused_matrix[i][0])
                original_old_nodes.append(fused_matrix[i][0])
                original_nodes +=1
            if fused_matrix[i][1] not in all_nodes:
                all_nodes.append(fused_matrix[i][1])
                original_old_nodes.append(fused_matrix[i][1])
                original_nodes +=1

    # % of Sequential and Topological and Not Matched
    totals = 0
    sequen = 0
    topolo = 0
    notmat = 0
    sequen2_listed = []
    topolo2_listed = []
    nodes_onto2_assigned = []
    for i in range(len(equivalent_matrix)):
        totals += 1
        if equivalent_matrix[i][4] == 'Sequential':
            sequen += 1
            if equivalent_matrix[i][1] not in sequen2_listed and equivalent_matrix[i][1] != '0' \
                    and equivalent_matrix[i][1] != 0:
                sequen2_listed.append(equivalent_matrix[i][1])
        elif equivalent_matrix[i][4] == 'Topological':
            topolo += 1
            if equivalent_matrix[i][1] not in topolo2_listed and equivalent_matrix[i][1] != '0' \
                    and equivalent_matrix[i][1] != 0:
                topolo2_listed.append(equivalent_matrix[i][1])
        elif equivalent_matrix[i][4] == 'Not Matched':
            notmat += 1
        nodes_onto2_assigned.append(equivalent_matrix[i][1])

    # Find how many ontology2 nodes don't be at the final ontology fused
    sequen2 = sequen2_listed.__len__()
    topolo2 = topolo2_listed.__len__()
    notmat2 = nodes2.__len__() - (sequen2 + topolo2)
    graph = graph2
    nodes = np.array(graph.nodes)
    nodes_left = []
    for i in range(len(nodes)):
        if nodes[i] not in all_nodes and nodes[i] not in nodes_onto2_assigned and nodes[i] not in nodes_left:
            nodes_left.append(nodes[i])
        if nodes[i] not in all_nodes and nodes[i] not in nodes_onto2_assigned and nodes[i] not in nodes_left:
            nodes_left.append(nodes[i])

    nodes_onto2_sort = len(list(set(nodes_onto2_assigned)))

    if return_values:
        return sequen, topolo, notmat, totals, edges_add, nodes_add, original_edges, original_nodes, nodes_onto2_sort,\
               sequen2, topolo2, notmat2
    elif not return_values:

        # Print results:
        print('added as a sequential: ', sequen)
        p_sequen = "{0:.3f}".format(sequen/totals*100)
        print('% of sequential match: ', p_sequen, '%')
        print('added as a topological-local sequential: ', topolo)
        p_topolo = "{0:.3f}".format(topolo / totals*100)
        print('% of topological match: ', p_topolo, '%')
        print('Not Matched: ', notmat)
        p_notmat = "{0:.3f}".format(notmat / totals*100)
        print('% of not matched: ', p_notmat, '%')

        print('Edges added: ', edges_add)
        print('edges grow in a ', "{0:.3f}".format(edges_add / original_edges * 100), '%')
        print('Nodes added: ', nodes_add)
        print('nodes grow in a ', "{0:.3f}".format(nodes_add / original_nodes * 100), '%')
        return


def quality_controls_V2(equivalent_matrix, fused_matrix, graph1, graph2):
    '''
    :param equivalent_matrix: Matrix of equivalence obtained during fusing ontology
    :param fused_matrix: final matrix from the ontology fusing process
    :return: Shows the results of quality control
    '''
    # Number of nodes and edges added
    nodes1 = list(graph1.nodes)
    nodes2 = list(graph2.nodes)
    edges1 = list(graph1.edges)
    # edges2 = list(graph2.edges)

    # make a edges structure
    descendant_relations = []
    for i in range(len(fused_matrix)):
        descendant_relations.append([fused_matrix[i][0], fused_matrix[i][1]])
    resulted_graph = treeComp.buildDiGraph(descendant_relations)
    nodes_result = list(resulted_graph.nodes)

    # % of Sequential and Topological and Not Matched
    totals = 0
    sequen = 0
    topolo = 0
    notmat = 0
    sequen2_listed = []
    topolo2_listed = []
    nodes_onto2_assigned = []
    for i in range(len(equivalent_matrix)):
        totals += 1
        if equivalent_matrix[i][4] == 'Sequential':
            sequen += 1
            if equivalent_matrix[i][1] not in sequen2_listed and equivalent_matrix[i][1] != '0' \
                    and equivalent_matrix[i][1] != 0:
                sequen2_listed.append(equivalent_matrix[i][1])
        elif equivalent_matrix[i][4] == 'Topological':
            topolo += 1
            if equivalent_matrix[i][1] not in topolo2_listed and equivalent_matrix[i][1] != '0' \
                    and equivalent_matrix[i][1] != 0:
                topolo2_listed.append(equivalent_matrix[i][1])
        elif equivalent_matrix[i][4] == 'Not Matched':
            notmat += 1
        nodes_onto2_assigned.append(equivalent_matrix[i][1])

    # Find how many ontology2 nodes don't be at the final ontology fused
    sequen2 = sequen2_listed.__len__()
    topolo2 = topolo2_listed.__len__()
    notmat2 = nodes2.__len__() - (sequen2 + topolo2)
    graph = graph2
    nodes = np.array(graph.nodes)
    nodes_left = []
    for i in range(len(nodes)):
        if nodes[i] not in nodes_result and nodes[i] not in nodes_onto2_assigned and nodes[i] not in nodes_left:
            nodes_left.append(nodes[i])
        if nodes[i] not in nodes_result and nodes[i] not in nodes_onto2_assigned and nodes[i] not in nodes_left:
            nodes_left.append(nodes[i])

    nodes_onto2_sort = len(list(set(nodes_onto2_assigned)))

    return sequen, topolo, notmat, totals, len(fused_matrix) - len(edges1), len(nodes_result) - len(nodes1), \
           len(edges1), len(nodes1), nodes_onto2_sort, sequen2, topolo2, notmat2


def circular_graphing(data, name, directory, result=True, color=None, parallelization=False, g1=None, equiv=None):

    coord.take_coordinates(data, result, color, g1, equiv)
    coordin = coord.parse_documents_coordinates(data, result)
    co.circular_graph(coordin, directory + name)
    if not parallelization:
        root_dir = os.path.realpath(__file__)
        project_path = root_dir + os.sep + 'fontcell_files'
        os.remove(project_path + os.sep + 'graph.gv')
        os.remove(project_path + os.sep + 'graph.gv.pdf')
        os.remove(project_path + os.sep + 'graph.gv.plain')
        os.remove(project_path + os.sep + 'graph_viz_coord.ods')
    return


def generate_subgraphs(ontology, nodes, windows):
    subgraphs = np.zeros(len(nodes), dtype=object)
    for j in range(len(nodes)):
        subgraphs[j] = treeComp.subGraphGenerator(ontology, nodes[j], windows)
    return subgraphs


def set_True_root(graph1, root):
    root_final = root
    nodes = list(graph1.nodes)
    for i in range(len(nodes)):
        if eval(nodes[i])[0] == root:
            root_final = nodes[i]

    return root_final


def fuseOntologies3(onto1, onto2
                    , name1, name2
                    , onto1_path=''
                    , onto1_fuseclasses=True
                    , onto2_fuseclasses=True
                    , onto1_restriction=False
                    , onto2_restriction=False
                    , onto1_list_clear=None
                    , onto2_list_clear=None
                    , synonyms=False
                    , text_process=False
                    , split_from1=None
                    , split_since1=None
                    , split_from2=None
                    , split_since2=None
                    , windowsize=4
                    , globalthreshold=0.85
                    , localthreshold=0.7
                    , constriction_threshold=0.0
                    , topological_threshold=0.0
                    , save_internals=True
                    , parallelization=False
                    , OBO_format_result=True
                    , save_internals_equiv=True
                    , draw_circular=True
                    , topological_test=True
                    , topological_cosine=False
                    , topological_pearson=False
                    , topological_euclidean=False
                    , output_folder=None):

    # generate names
    # r_path = os.path.dirname(os.path.realpath(__file__))
    root_dir = os.path.dirname(os.path.realpath(__file__))
    path_to_doc = root_dir + os.sep + 'fontcell_files' + os.sep
    # path_to_img = r_path + os.sep + 'images' + os.sep
    if output_folder is None:
        output_folder = path_to_doc
    comm = MPI.COMM_WORLD
    first_partA = name1[0:1]

    first_partB = name2[0:1]

    add_name1 = str(first_partA)[0] + '_' + str(first_partB)[0] + '_S' + ''.join(str(globalthreshold).split('.'))
    if topological_test:
        add_name2 = '_LS' + ''.join(str(localthreshold).split('.'))
    else:
        add_name2 = '_CT' + ''.join(str(constriction_threshold).split('.'))
    add_name3 = '_W' + str(windowsize)

    sequential_name = 'seq_matrix_' + add_name1 + '.csv'
    # topological_name = folder + 'top_matrix_' + add_name1 + add_name3 + '.csv'
    equivalent_name = 'equiv_dictio_' + add_name1 + add_name2 + add_name3 + '.ods'
    result_name = 'f_graphs_' + add_name1 + add_name2 + add_name3 + '.ods'

    # print('import the ontologies')
    # if onto1_fusedproduct:
    #     # print('do something')
    #     onto1 = fromOutputFusedOntology_to_readableOntology(import_ods(onto1_path))
    # else:
    #     onto1 = import_ods(onto1_path)
    # onto2 = import_ods(onto2_path)

    print('make a graph for each ontology')
    # T ~ 3 mins
    O1, secondary1 = treeComp.makeDigraph_fromOntology(onto1, onto1_fuseclasses, onto1_restriction)
    O2, secondary2 = treeComp.makeDigraph_fromOntology(onto2, onto2_fuseclasses, onto2_restriction)
    nodes1 = list(map(lambda a: eval(a), O1.nodes))
    nodes2 = list(map(lambda a: eval(a), O2.nodes))
    # Pre-generate sub-graph
    if parallelization:
        comm = MPI.COMM_WORLD

        if comm.rank == 0:
            idx1 = [int(i) for i in np.linspace(0, len(nodes1), comm.size+1)]
            nodes1 = [nodes1[idx1[i]:idx1[i+1]] for i in range(len(idx1)-1)]

            idx2 = [int(i) for i in np.linspace(0, len(nodes2), comm.size + 1)]
            nodes2 = [nodes2[idx2[i]:idx2[i + 1]] for i in range(len(idx2) - 1)]

        else:
            nodes1, nodes2 = None, None


        nodes1_p = comm.scatter(nodes1)
        nodes2_p = comm.scatter(nodes2)
        subgraphs1_r = generate_subgraphs(O1, nodes1_p, windowsize)
        subgraphs2_r = generate_subgraphs(O2, nodes2_p, windowsize)
        subgraphs1 = comm.allgather(subgraphs1_r)
        subgraphs2 = comm.allgather(subgraphs2_r)

        xxx = []
        yyy = []
        for i in subgraphs1:
            xxx += list(i)
        for j in subgraphs2:
            yyy += list(j)

        subgraphs1, subgraphs2 = xxx, yyy

        comm.Barrier()
        # print('Debugging')
        # print(type(subgraphs1))
        # print(subgraphs1)
            # print(list(subgraphs2[0].nodes))
    else:
        print('Pre-generate sub-graph')
        subgraphs1 = generate_subgraphs(O1, nodes1, windowsize)
        subgraphs2 = generate_subgraphs(O2, nodes2, windowsize)

    # root = set_True_root(O1, root_1)
    # subgraphs1 = np.zeros(len(nodes1), dtype=object)
    # for j in range(len(nodes1)):
    #     subgraphs1[j] = treeComp.subGraphGenerator(O1, nodes1[j], windowsize)
    #
    # subgraphs2 = np.zeros(len(nodes2), dtype=object)
    # for j in range(len(nodes2)):
    #     subgraphs2[j] = treeComp.subGraphGenerator(O2, nodes2[j], windowsize)

    # SEQUENTIAL MATRIX #
    print('generate sequential matrix')
    try:
        sequential = doc.import_csv(path_to_doc + sequential_name)
    except:
        # graph1, graph2, text_process = False, synonyms = False, word_list_to_clear1 = None,
        # word_list_to_clear2 = None, split_from1 = None, split_since1 = None, split_from2 = None, split_since2 = None
        sequential = gen_text_sequential_matrix(O1, O2, text_process, synonyms, onto1_list_clear, onto2_list_clear,
                                                split_from1, split_since1, split_from2, split_since2)
        if save_internals:
            # SAVING SEQUENTIAL #
            doc.save_csv(sequential, path_to_doc + sequential_name)

    # EQUIVALENT #
    print('Set equivalent nodes')
    # if topological T ~ +20 min
    # if not topological T ~
    try:
        equivalent = doc.import_ods(path_to_doc + equivalent_name)
    except:
        if parallelization:
            equivalent = equivalent_dictionaryV4_parallelized(comm, O1, O2, sequential, globalthreshold, localthreshold
                                                              , subgraphs1,subgraphs2, windowsize=windowsize
                                                              , simtopo=topological_test
                                                              , constriction_threshold=constriction_threshold
                                                              , topological_cosine=topological_cosine
                                                              , topological_pearson=topological_pearson
                                                              , topological_euclidean=topological_euclidean
                                                              , topological_threshold=topological_threshold)
            comm.Barrier()
        else:
            equivalent = equivalent_dictionaryV4(O1, O2, sequential, globalthreshold, localthreshold, subgraphs1
                                                 , subgraphs2, windowsize=windowsize, simtopo=topological_test
                                                 , constriction_threshold=constriction_threshold
                                                 , topological_cosine=topological_cosine
                                                 , topological_pearson=topological_pearson
                                                 , topological_euclidean=topological_euclidean
                                                 , topological_threshold=topological_threshold)
        if save_internals_equiv:
            # SAVE EQUIVALENT #
            doc.save_ods(equivalent, path_to_doc + equivalent_name)
    if parallelization:
        rank = comm.Get_rank()
        print(rank)
        if rank == 0:
            print(equivalent.__len__())
            # FUSING GRAPHS #
            print('fuse graphs')
            result, type_dict = fuse_graphs(O1, O2, equivalent, root=None)
            doc.save_ods(result, path_to_doc + result_name)
            # save_ods()
            if OBO_format_result:
                if not secondary1:
                    print('DISCLAIMER: can not perform a OBO format: Ontology 1 input as not IDs')
                else:
                    obo.OBOFormat(result, doc.import_txt(onto1_path), secondary1,
                              file_name=output_folder + first_partA + '_' + first_partB)
            if draw_circular:
                circular_graphing(O1, 'ontology1_graph', directory=output_folder, result=False, color='orange',  parallelization=parallelization)
                circular_graphing(O2, 'ontology2_graph', directory=output_folder, result=False, color='lightskyblue', parallelization=parallelization)
                circular_graphing(result, 'fused_ontology_graph',  directory=output_folder, parallelization=parallelization, g1=O1, equiv=type_dict)
                topo_state = set_topologicalstate(topological_test, topological_cosine, topological_pearson, topological_euclidean)

                create_HTML_w_results('ontology1_graph.html', 'ontology2_graph.html','fused_ontology_graph.html',
                                      name1, name2, equivalent, result, O1,  O2, topo_state, globalthreshold,
                                      localthreshold, constriction_threshold, topological_threshold, OBO_format_result,
                                      file_name_output=output_folder, obo_name=first_partA + '_' + first_partB)
            return
        else:
            return
    else:
        # FUSING GRAPHS #
        print('fuse graphs')
        result, type_dict = fuse_graphs(O1, O2, equivalent, root=None)
        doc.save_ods(result, path_to_doc + result_name)
        # save_ods()
        if OBO_format_result:
            obo.OBOFormat(result, doc.import_txt(onto1_path), secondary1, output_folder + first_partA + '_' + first_partB)
        if draw_circular:
            circular_graphing(O1, 'ontology1_graph', directory=output_folder, result=False, color='orange',
                              parallelization=parallelization)
            circular_graphing(O2, 'ontology2_graph', directory=output_folder, result=False, color='lightskyblue',
                              parallelization=parallelization)
            circular_graphing(result, 'fused_ontology_graph', directory=output_folder,
                              parallelization=parallelization, g1=O1, equiv=type_dict)
            topo_state = set_topologicalstate(topological_test, topological_cosine, topological_pearson,
                                              topological_euclidean)
            create_HTML_w_results('ontology1_graph.html','ontology2_graph.html', 'fused_ontology_graph.html', name1,
                                  name2, equivalent, result, O1, O2, topo_state, globalthreshold, localthreshold,
                                  constriction_threshold, topological_threshold, OBO_format_result,
                                  file_name_output=output_folder, obo_name=first_partA + '_' + first_partB)
        return


def create_HTML_w_results(circle1, circle2, circlemixed, onto1name1, onto2name1, equivalent, result, graph1, graph2,
                          topo_state, sim_threshold, localthreshold, cons_threshold, topological_threshold, OBO,
                          file_name_output, obo_name):
    root_dir = os.path.dirname(os.path.realpath(__file__))
    current_path_img = root_dir + os.sep + 'fontcell_' + os.sep

    sequen, topolo, notmat, totals, edges_add, nodes_add, original_edges, original_nodes, nodesonto2_assigned,\
    sequen2, topolo2, notmat2 = quality_controls_V2(equivalent, result, graph1, graph2)
    nodesonto2 = len(list(graph2.nodes))
    onto1name = clean_onto_name(onto1name1)
    onto2name =clean_onto_name(onto2name1)
    style1 = h.add_style('', type='h', number=1, style='color: #000000;')
    style2 = h.add_style(style1, type='h', number=2, style='color: #ffffff;')
    style3 = h.add_style(style2, type='.block1', number='', style='background-color: #000099;')
    style4 = h.add_style(style3, type='.block2', number='', style='background-color: #000099;')
    styles = h.add_style(style4, type='h', number=3, style='color: #ffffff;')

    fig0 = h.insert_image(current_path_img + 'logo.png', width=150, height=150, align='left')
    heading = h.heading(fig0 + 'FOntCell <br> Fusion of ' + onto1name + ' and ' + onto2name, number=1)
    heading2 = h.add_box(h.heading('Interactive circular Directed Acyclic Graphs (DAGs) of (a) ' + onto1name
                                   + ', (b) ' + onto2name + ' and (c) Fused ontologies', number=2), type='block1')

    link1 = h.insert_link(file_name_output + circle1, '(a) DAG of ' + onto1name + ' ontology (nodes in orange)')
    link2 = h.insert_link(file_name_output + circle2, '(b) DAG of ' + onto2name + ' ontology (nodes in blue)')
    link3 = h.insert_link(file_name_output + circlemixed, '(c) DAG of the Fused ontology (nodes in orange and blue)')

    line0 = h.paragraph('The ontology labels associated to the nodes appear when hovering over the nodes.')
    line1 = h.paragraph('The ontology labels associated to the nodes appear when hovering over the nodes.')

    first_part = heading + h.blankspace() + h.blankspace() + h.blankspace() + h.blankspace() + h.blankspace() + \
                 heading2 + link1 + h.blankspace() + link2 + h.blankspace() + link3 + h.blankspace() + line0 + line1

    heading3_1 = h.add_box(h.heading('Parameters of the FOntCell fusion algorithm', number=3), type='block2')
    line3_1_1 = ['Sequence similarity threshold &Theta;<sub>S</sub>: ' + str(sim_threshold)]
    line3_1_2 = ['Topology metric: ' + str(topo_state)]
    if topo_state != 'constriction':
        line3_1_3 = ['Sequence local similarity threshold &Theta;<sub>SL</sub>: '
                     + str(localthreshold)]
        line3_1_4 = ['Topology similarity threshold &Theta;<sub>T</sub>: '
                     + str(topological_threshold)]
        list3_1 = heading3_1 + h.htmllist(line3_1_1 + line3_1_2 + line3_1_3 + line3_1_4)
    else:
        line3_1_3 = ['Constriction threshold &Theta;<sub>C</sub>: ' + str(cons_threshold)]
        list3_1 = heading3_1 + h.htmllist(line3_1_1 + line3_1_2 + line3_1_3)

    heading3_2 = h.add_box(h.heading('Statistics of the input ontologies', number=3), type='block2')
    line3_2_1 = ['Number of nodes of ' + onto1name + ' ontology: ' + str(original_nodes)]
    line3_2_2 = ['Number of edges of ' + onto1name + ' ontology: ' + str(original_edges)]
    line3_2_3 = ['Number of nodes of ' + onto2name + ' ontology: ' + str(nodes_add)]
    line3_2_4 = ['Number of edges of ' + onto2name + ' ontology: ' + str(edges_add)]
    list3_2 = heading3_2 + h.htmllist(line3_2_1 + line3_2_2 + line3_2_3 + line3_2_4)

    heading3_3 = h.add_box(h.heading('Statistics of the fused ontology', number=3), type='block2')
    heading3_3_1 = h.heading('Statistics of the fusion by sequence similarity', number=4)
    line3_3_1_1 = ['Number of nodes added to ' + onto1name + ' by sequence similarity: ' + str(sequen)]
    line3_3_1_2 = ['Percentage of nodes (in relation to the number of nodes of ' \
                  + onto1name + ' ontology) added to '\
                  + onto1name + ' by sequence similarity: '\
                  + str("{0:.2f}".format(sequen / original_nodes * 100)) + '%']

    list3_3_1= heading3_3_1 + h.htmllist(line3_3_1_1 + line3_3_1_2)  # + line3_3_1_3 + line3_3_1_4)

    heading3_3_2 = h.heading('Statistics of the fusion by topology similarity', number=4)
    line3_3_2_1 = ['Number of nodes added to ' + onto1name + ' by topology similarity: ' + str(topolo)]
    line3_3_2_2 = ['Percentage of nodes added to ' \
                  + onto1name + ' (in relation to the number of nodes of '\
                  + onto1name + ' ontology) by topology similarity: '\
                  + str("{0:.2f}".format(topolo / original_nodes * 100)) + '%']

    list3_3_2= heading3_3_2 + h.htmllist(line3_3_2_1 + line3_3_2_2)  # + line3_3_2_3 + line3_3_2_4)

    heading3_3_3 = h.heading('Statistics of the fusion of non-matched nodes', number=4)
    line3_3_3_1 = ['Number of nodes in ' + onto1name + ' non-matched in ' + onto2name + ': ' + str(notmat)]
    line3_3_3_2 = ['Percentage of nodes in '\
                  + onto1name + ' not matched in '\
                  + onto2name + ' (in relation to the number of nodes of '\
                  + onto1name + ' ontology): ' + str("{0:.2f}".format(notmat / original_nodes * 100)) + '%']
    line3_3_3_3 = ['Number of nodes in ' + onto2name + ' non-matched in ' + onto1name + ': ' + str(nodesonto2_assigned)]

    line3_3_3_4 = ['Percentage of nodes in '\
                  + onto2name + ' not matched in '\
                  + onto1name + ' (in relation to the number of nodes of ' \
                  + onto2name + ' ontology): '\
                  + str("{0:.2f}".format(nodesonto2_assigned / nodesonto2* 100)) + '%']
    list3_3_3 = heading3_3_3 + h.htmllist(line3_3_3_1 + line3_3_3_2 + line3_3_3_3 + line3_3_3_4)

    heading3_3_4 = h.heading('Statistics of the fusion by sequence and topology similarity', number=4)
    line3_3_4_1 = ['Number of nodes added in total (by sequence similarity and by topology similarity): ' + str(nodes_add)]
    line3_3_4_2 = ['Percentage of nodes added in total (by sequence similarity and by topology similarity): ' \
                  + str("{0:.2f}".format(nodes_add / original_nodes * 100)) + '%']
    line3_3_4_3 = ['Number of edges added in total (by sequence similarity and by topology similarity): ' + str(edges_add)]
    line3_3_4_4 = ['Percentage of edges added in total (by sequence similarity and by topology similarity): ' \
                  + str("{0:.2f}".format(edges_add / original_edges * 100)) + '%']
    list3_3_4= heading3_3_4 + h.htmllist(line3_3_4_1 + line3_3_4_2 + line3_3_4_3 + line3_3_4_4)

    second_part = list3_1 + list3_2 + heading3_3 + list3_3_1 + list3_3_2 + list3_3_3 + list3_3_4 + h.blankspace()

    if OBO:
        header_for_OBO = h.add_box(h.heading('OBO Format of the ontology Fused', number=3), type='block2')
        line_for_OBO = h.paragraph('Fused Ontology from ' + onto1name + ' and ' + onto2name + ': ')
        link4 = header_for_OBO + line_for_OBO + h.insert_link(file_name_output + obo_name + '.owl', 'here')
    else:
        link4 = ''

    heading4 = h.add_box(h.heading('Changes of number of nodes and edges during fusion and Euler-Venn diagram of the Fused ontology', number=2), type='block1')
    line4_1 = 'Percentages of nodes and edges changed (in relation to ' \
              + onto1name + ') during the different phases of fusion.'

    make.make_relatedfigures(graph1, graph2, onto1name, onto2name, sequen, topolo, notmat, nodes_add, original_nodes,
                             sequen2,topolo2, notmat2, donut_name=file_name_output + 'donut_diagram.png',
                             square_name=file_name_output + 'fusion_square_plot.png', donut=True, squares=True)

    fig1 = h.align(h.insert_image(file_name_output + 'donut_diagram.png', width=640, height=480), 'left')

    line4_2 = 'Euler-Venn diagram of the common nodes between ' + onto1name + ' and ' + onto2name

    fig2 = h.insert_image(file_name_output + 'fusion_square_plot.png', width=640, height=480)

    third_part = link4 + heading4 + line4_1 + fig1 + line4_2 + h.blankspace() + fig2

    body = h.body(first_part + second_part + third_part)

    full_html = h.html(styles + body)

    doc.save_txt(full_html, file_name_output + 'FOntCell_' + onto1name + '_' + onto2name + '.html')

    return


def set_topologicalstate(simtopo, topological_cosine, topological_pearson, topological_euclidean):
    if not simtopo:
        topologicalstate = 'constriction'
    else:
        if topological_cosine:
            topologicalstate = 'cosine'
        elif topological_pearson:
            topologicalstate = 'Pearson'
        elif topological_euclidean:
            topologicalstate = 'euclidean'
        else:
            topologicalstate = 'Blondel'
    return topologicalstate


def clean_onto_name(name):
    listed_name = list(name)
    new_name = ''.join(list(filter(lambda a: a.isalpha(), listed_name)))
    return new_name



'''
From the fusion resulted 'ontology' create a .txt with the new classes added in OBO format. CELDA fingerprint
'''

#############################
#         Libraries         #
#############################
import numpy as np
from . import doc_managing as doc
#############################
#         FUNCTIONS         #
#############################


def OBOFormat(resulted_ontology, old_ontology, IDs_dictionary, file_name='ontology_fused'):
    '''
    Main function to run the
    '''
    # We have to set [ID, label, synonyms, father]

    # Split ontology in two slice to put in the middle the new classes
    splitting_site = old_ontology.index('    // Classes\n') + 5
    old_ontology1 = old_ontology[:splitting_site]
    old_ontology2 = old_ontology[splitting_site:]

    # Create a dictionary from IDs_dictionary and a list of IDs to set the max id number
    id_list = []
    dictionary_of_IDs = {}
    IDs_dictionary_listed = list(IDs_dictionary)
    for i in range(len(IDs_dictionary_listed)):
        label =IDs_dictionary_listed[i]
        id = IDs_dictionary[IDs_dictionary_listed[i]]
        id_list.append(int(id.split('_')[1]))
        dictionary_of_IDs.update({label:id})
    main_ID = 'Fused#' + id.split('_')[0] + '_'
    number_characters = len(id.split('_')[1])
    max_ID = max(id_list)

    # Order nodes to son-father set
    edge = list(np.zeros([len(resulted_ontology), 2], dtype=object))

    for i in range(len(edge)):
        edge[i] = [eval(resulted_ontology[i][1]), eval(resulted_ontology[i][0])]

    dict_nodes_S_to_F = {}
    for k, v in edge:
        dict_nodes_S_to_F.setdefault(k, []).append(v)

    # split into new and old nodes
    old_nodes = []
    new_nodes = []
    for i in range(len(resulted_ontology)):
        if resulted_ontology[i][2] == 'new':
            if eval(resulted_ontology[i][0]) not in new_nodes and eval(resulted_ontology[i][0]) not in old_nodes:
                new_nodes.append(eval(resulted_ontology[i][0]))
            if eval(resulted_ontology[i][1]) not in new_nodes and eval(resulted_ontology[i][1]) not in old_nodes:
                new_nodes.append(eval(resulted_ontology[i][1]))
        else:
            if eval(resulted_ontology[i][0]) not in old_nodes:
                old_nodes.append(eval(resulted_ontology[i][0]))
            if eval(resulted_ontology[i][1]) not in old_nodes:
                old_nodes.append(eval(resulted_ontology[i][1]))
    print()

    # Set IDs to the new nodes (in a dictionary) updating: dictionary_of_IDs
    for i in range(len(new_nodes)):
        # generate the new ID
        max_ID += 1
        current_ID = main_ID + setID_format(max_ID, number_characters)
        dictionary_of_IDs.update({new_nodes[i]:current_ID})

    # Build an array for these new nodes [ID, label, synonyms, father]

    for i in range(len(new_nodes)):
        # Take the label
        current_label = new_nodes[i][0]
        # Perform synonymous list
        current_synonyms = new_nodes[i][1:]
        if current_synonyms.__len__() == 0:
            current_synonyms = None
        # take father
        current_father = list(map(lambda a:dictionary_of_IDs[a], dict_nodes_S_to_F[new_nodes[i]]))
        if current_father.__len__() == 0:
            current_father = None
        #take ID
        current_ID = dictionary_of_IDs[new_nodes[i]]

        # TheArray[i] = [current_ID, current_label, current_synonyms, current_father]

        current_class = write_class(current_ID, current_label, current_synonyms, current_father)
        for i in range(len(current_class)):
            old_ontology1.append(current_class[i])

    ontology_formated = old_ontology1 + old_ontology2
    named_as = file_name + '.owl'
    doc.save_txt(ontology_formated, named_as)

    return


def write_class(ID, label, synonyms=None, father=None):
    '''
    write a class oin OBO format (inspired by CELDA ontology)
    :param ID: str ID of class
    :param label: str class label
    :param synonyms: list synonyms
    :param father: str ID of class father
    :return: a formated class
    '''

    if synonyms != None and father != None:

        all_fathers = ''
        for i in range(len(father)):
            all_fathers= all_fathers + father[i]

        new_class = ['\n',
                     '\n',
                     '\n',
                     '     <!--#' + ID + '-->\n',
                     '\n',
                     '     <owl:Class rdf:about="#' + ID +'">\n',
                     '         <rdfs:label>'+ label + '</rdfs:label>\n',
                     '         <rdfs:subClassOf>\n',
                     '         <owl:Restriction>\n',
                     '             <owl:onProperty rdf:resource="&purl;obo/develops_from"/>\n',
                     '             <owl:someValuesFrom rdf:resource="' + all_fathers + '"/>\n',
                     '         </owl:Restriction>\n']
        # loop for synonyms
        for i in range(len(synonyms)):
            new_class.append('         <ru-meta:synonym xml:lang="en">' + synonyms[i] + '</ru-meta:synonym>\n')

        new_class.append('        </rdfs:subClassOf>\n')
        new_class.append('     </owl:Class>\n')


    elif synonyms != None and father == None:
        new_class = ['\n',
                     '\n',
                     '\n',
                     '    <!--#' + ID + '-->\n',
                     '\n',
                     '    <owl:Class rdf:about="#' + ID + '">\n',
                     '        <rdfs:label>' + label + '</rdfs:label>\n']
        # loop for synonyms
        for i in range(len(synonyms)):
            new_class.append('        <ru-meta:synonym xml:lang="en">' + synonyms[i] + '</ru-meta:synonym>\n')
        new_class.append('    </owl:Class>\n')

    elif synonyms == None and father != None:

        all_fathers = ''
        for i in range(len(father)):
            all_fathers = all_fathers + father[i]

        new_class = ['\n',
                     '\n',
                     '\n',
                     '    <!--#' + ID + '-->\n',
                     '\n',
                     '    <owl:Class rdf:about="#' + ID + '">\n',
                     '        <rdfs:label>' + label + '</rdfs:label>\n',
                     '        <rdfs:subClassOf >\n',
                     '        <owl:Restriction>\n',
                     '            <owl:onProperty rdf:resource="&purl;obo/develops_from"/>\n',
                     '            <owl:someValuesFrom rdf:resource="' + all_fathers + '"/>\n',
                     '        </owl:Restriction>\n',
                     '        </rdfs:subClassOf>\n',
                     '    </owl:Class>\n']

    else:
        new_class = ['\n',
                     '\n',
                     '\n',
                     '    <!--#' + ID + '-->\n',
                     '\n',
                     '    <owl:Class rdf:about="#' + ID + '">\n',
                     '        <rdfs:label>' + label + '</rdfs:label>\n',
                     '    </owl:Class>\n']

    return new_class


def setID_format(number, characters_len):
    '''
    Generates a str ready to attach to the 'main core' of the ID
    :param number: int of the ID
    :param characters_len: number of characters that the 'int part' of the ID has to be
    :return: 'int part' of the ID
    '''
    id = ''
    for i in range(characters_len - len(str(number))):
        id = id + '0'
    id_to = id + str(number)
    return id_to



# Libraries #
import os
from . import ontology_fusion as of
from mpi4py import MPI
from . import doc_managing as doc
# ######### #

def read_variables(variable):
    try:
        vari = eval(variable)
    except:
        vari = variable
    return vari

# Read the configurations
def import_and_delete():
    root_dir = os.path.dirname(os.path.realpath(__file__))
    project_path = root_dir + os.sep + 'fontcell_files' + os.sep

    list_of_variables = doc.import_txt(project_path + 'parallelization_config.txt')
    ontology1 = doc.import_csv(project_path + 'onto1_for_parallel.csv')
    ontology2 = doc.import_csv(project_path + 'onto2_for_parallel.csv')

    onto_name1 = read_variables(list_of_variables[0])
    onto_name2 = read_variables(list_of_variables[1])
    file1 = read_variables(list_of_variables[2])
    onto1_fuseclasses = read_variables(list_of_variables[3])
    onto2_fuseclasses = read_variables(list_of_variables[4])
    onto1_restriction = read_variables(list_of_variables[5])
    onto2_restriction = read_variables(list_of_variables[6])
    onto1_list_clear = read_variables(list_of_variables[7])
    onto2_list_clear = read_variables(list_of_variables[8])
    synonyms = read_variables(list_of_variables[9])
    text_process = read_variables(list_of_variables[10])
    split_from1 = read_variables(list_of_variables[11])
    split_since1 = read_variables(list_of_variables[12])
    split_from2 = read_variables(list_of_variables[13])
    split_since2 = read_variables(list_of_variables[14])
    windowsize = read_variables(list_of_variables[15])
    globalthreshold = read_variables(list_of_variables[16])
    localthreshold = read_variables(list_of_variables[17])
    save_internals = read_variables(list_of_variables[18])
    parallelization = True
    OBO_format_result = read_variables(list_of_variables[20])
    save_internals_equiv = read_variables(list_of_variables[21])
    folder = read_variables(list_of_variables[22])
    draw_circular = read_variables(list_of_variables[23])
    topological_test = read_variables(list_of_variables[24])
    constriction_threshold = read_variables(list_of_variables[25])
    topological_cosine = read_variables(list_of_variables[26])
    topological_pearson = read_variables(list_of_variables[27])
    topological_euclidean = read_variables(list_of_variables[28])
    topological_threshold = read_variables(list_of_variables[29])
    root = read_variables(list_of_variables[30])

    return ontology1, ontology2, onto_name1, onto_name2, file1,onto1_fuseclasses, onto2_fuseclasses\
        , onto1_restriction, onto2_restriction, onto1_list_clear, onto2_list_clear, synonyms\
        , text_process, split_from1, split_since1, split_from2, split_since2, windowsize\
        , globalthreshold, localthreshold, save_internals, parallelization, OBO_format_result\
        , save_internals_equiv, folder, draw_circular, topological_test, constriction_threshold, topological_cosine\
        , topological_pearson, topological_euclidean, topological_threshold, root


# Perform the fusion parallelizing
def ontology_fusion_parallelized():
    ontology1, ontology2, onto_name1, onto_name2, file1, onto1_fuseclasses, onto2_fuseclasses\
        , onto1_restriction, onto2_restriction, onto1_list_clear, onto2_list_clear, synonyms\
        , text_process, split_from1, split_since1, split_from2, split_since2, windowsize\
        , globalthreshold, localthreshold, save_internals, parallelization, OBO_format_result\
        , save_internals_equiv, folder, draw_circular, topological_test, constriction_threshold, topological_cosine\
        , topological_pearson, topological_euclidean, topological_threshold, root = import_and_delete()

    of.fuseOntologies3(ontology1, ontology2, onto_name1, onto_name2, onto1_path=file1,
                       onto1_fuseclasses=onto1_fuseclasses, onto2_fuseclasses=onto2_fuseclasses,
                       onto1_restriction=onto1_restriction, onto2_restriction=onto2_restriction,
                       onto1_list_clear=onto1_list_clear, onto2_list_clear=onto2_list_clear, synonyms=synonyms,
                       text_process=text_process, split_from1=split_from1, split_since1=split_since1,
                       split_from2=split_from2, split_since2=split_since2, windowsize=windowsize,
                       globalthreshold=globalthreshold, localthreshold=localthreshold,
                       constriction_threshold=constriction_threshold, topological_threshold=topological_threshold,
                       save_internals=save_internals, parallelization=parallelization,
                       OBO_format_result=OBO_format_result, save_internals_equiv=save_internals_equiv,
                       draw_circular=draw_circular, topological_test=topological_test,
                       topological_cosine=topological_cosine, topological_pearson=topological_pearson,
                       topological_euclidean=topological_euclidean)
    comm = MPI.COMM_WORLD
    rank = comm.Get_rank()
    if rank == 0:
        root_dir = os.path.dirname(os.path.realpath(__file__))
        project_path = root_dir + os.sep + 'fontcell_files' + os.sep
        os.remove(project_path + 'parallelization_config.txt')
        os.remove(project_path + 'onto1_for_parallel.csv')
        os.remove(project_path + 'onto2_for_parallel.csv')
        os.remove(project_path + 'graph.gv')
        os.remove(project_path + 'graph.gv.pdf')
        os.remove(project_path + 'graph.gv.plain')
        os.remove(project_path + 'graph_viz_coord.ods')
    return


if __name__ == '__main__':
    ontology_fusion_parallelized()



import numpy as np
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from math import sqrt, radians


def set_labeling_coordinates(wedges, length):
    label_coord = np.zeros([len(wedges), 2], dtype=float)

    # set the polar coordinates
    coordinates = take_polar_coord(wedges)
    mean_points = list(map(lambda a: np.mean(a), coordinates))
    label_polar_coord = [[float("{0:.2f}".format(radians(coordinates[i][0]))),
                          float("{0:.2f}".format(radians(mean_points[i])))] for i in range(len(mean_points))]

    # set the sector
    for i in range(len(label_polar_coord)):

        # 1st sector:
        if 0.0 <= label_polar_coord[i][1] < 90.0:
            label_coord[i] = [np.cos(label_polar_coord[i][1]) * length, np.sin(label_polar_coord[i][1]) * length]

        # 2nd sector:
        elif 90.0 <= label_polar_coord[i][1] < 180.0:
            label_coord[i] = [- np.cos(label_polar_coord[i][1]) * length, np.sin(label_polar_coord[i][1]) * length]

        # 3rd sector:
        elif 180.0 <= label_polar_coord[i][1] < 270.0:
            label_coord[i] = [- np.cos(label_polar_coord[i][1]) * length, - np.sin(label_polar_coord[i][1]) * length]

        # 4th sector
        elif 270.0 <= label_polar_coord[i][1] < 360.0:
            label_coord[i] = [np.cos(label_polar_coord[i][1]) * length, - np.sin(label_polar_coord[i][1]) * length]

    return label_coord


def take_polar_coord(wedges):
    coordinates = np.zeros([len(wedges), 2], dtype=float)
    for i in range(len(wedges)):
        coordinates[i] = [wedges[i].theta1, wedges[i].theta2]
    return coordinates


def make_relatedfigures(graph1, graph2, name1, name2, sequen, topolo, notmat, nodes_add, original_nodes, sequen2,
                        topolo2, notmat2, donut_name='donut_diagram.png', square_name='fusion_square_plot.png',
                        donut=False, squares=False):
    # set bar details:
    color1 = '#6495ED'
    color2 = '#8FBC8F'
    color3 = '#FF7F50'
    color4 = '#008080'

    if donut:

        sequen1 = sequen
        topolo1 = topolo
        notmat1 = notmat
        make_donut(sequen1, topolo1, notmat1, sequen2, topolo2, notmat2, color1, color2, color3, donut_name,
                   name1, name2)
    if squares:
        fusion_squares(list(graph1.nodes), list(graph2.nodes), sequen + topolo, original_nodes + nodes_add,
                       color1, color2, color3, color4, square_name, name1, name2)
    return


def make_donut(sequen1, topolo1, notmat1, sequen2, topolo2, notmat2, color1, color2, color3, donut_name,
               onto1_name, onto2_name):

    fig, ax = plt.subplots()
    size = 0.3
    textes1 = ['Sequential', 'Topology', 'Non-matched']
    # textes2 = [onto2_name + ' Sequential', onto2_name + ' ' + topo_state, onto2_name + ' Non-matched']
    array1 = np.array([(sequen1 + topolo1 + notmat1), sequen1, topolo1, notmat1])
    array2 = np.array([(sequen2 + topolo2 + notmat2), sequen2, topolo2, notmat2])
    inter_vals1 = np.interp(array1, (0, array1.max()), (0, 100))
    inter_vals2 = np.interp(array2, (0, array2.max()), (0, 100))
    vals1 = inter_vals1[1:]
    vals2 = inter_vals2[1:]
    # cmap = plt.get_cmap("tab20c")
    outer_colors = np.array([color1, color2, color3])
    inner_colors = np.array([color1, color2, color3])

    wedges1 = ax.pie(vals1, radius=1, colors=outer_colors, wedgeprops=dict(width=size, edgecolor='w'))

    wedges2 = ax.pie(vals2, radius=1-size, colors=inner_colors, wedgeprops=dict(width=size, edgecolor='w'))
    coord1 = set_labeling_coordinates(wedges1[0], size*3)
    coord2 = set_labeling_coordinates(wedges2[0], size*2)
    # for i in range(len(wedges1)):
    #     ax.annotate(str("{0:.2f}".format(vals1[i])))
    # for j in enumerate(len(wedges2)):
    #     ax.annotate(str("{0:.2f}".format(vals2[j])))
    ax.legend(wedges1[0], textes1,
              # title="Ingredients",
              loc="center left",
              bbox_to_anchor=(1, 0, 0.5, 1),
              frameon=False)
    plt.text(0, 0, onto2_name, horizontalalignment='center', verticalalignment='center')
    plt.text(0, size*3.1 + size*0.4, onto1_name, horizontalalignment='center', verticalalignment='center')
    [plt.text(coord1[i][0], coord1[i][1], "{0:.1f}".format(vals1[i]), horizontalalignment='center',
              verticalalignment='center') for i in range(len(coord1))]
    [plt.text(coord2[j][0], coord2[j][1], "{0:.1f}".format(vals2[j]), horizontalalignment='center',
              verticalalignment='center') for j in range(len(coord2))]
    # plt.legend((color1, color2, color3, inner_colors[0], inner_colors[0]),
    #            ('Sequential', 'Topological', 'Not matched', 'Added', 'Original'))
    plt.savefig(donut_name)
    plt.close()
    return


def fusion_squares(onto1, onto2, common, resultant, color1, color2, color3, color4, fig_name, name1, name2, r=0.6):
    # set the minor 'x' and major 'y'

    if len(onto1) > len(onto2):
        a = len(onto2)
        b1 = len(onto1)
        color_a = color1
        color_b = color2
        label1 = name1
        label2 = name2
    else:
        a = len(onto1)
        b1 = len(onto2)
        label1 = name2
        label2 = name1
        color_b = color1
        color_a = color2

    # check if the minor is 0.2 smaller than the major
    if 0.8*b1 <= a:
        c1 = common * (r-0.2)
        result = resultant * (r-0.2)
        a1 = a * r
        base_a = sqrt(a1)
        height_a = sqrt(a1)
        base_b = sqrt(b1)
        height_b = sqrt(b1)
        base_c = (c1 / height_a)
        height_c = height_a
    else:
        c1 = common
        result = resultant
        # take the borders length
        base_a = sqrt(a)
        height_a = base_a
        base_b = sqrt(b1)
        height_b = base_b
        base_c = c1 / height_a
        height_c = height_a

    # base_c = c1 / height_a
    base_result = (result - (base_a*height_a) + (base_c * height_c)) / height_b
    # Z = border_x * border_z
    # set the start coordinates for x and y
    start_a = ((height_b - height_a) / 2)
    start_b = base_a - base_c
    # start_result = start_y
    # create figures and rectangles
    # fig,ax = plt.subplots()
    fig2 = plt.figure()
    ax = fig2.add_subplot(111, aspect='equal')
    height_max = max(height_b, height_a, height_c)
    base_max = max(base_b, base_a, base_c)
    y_limit = height_max + height_max * 0.1
    x_limit = start_b + base_max + base_max * 0.1
    ax.set_xlim([0, x_limit])
    ax.set_ylim([0, y_limit])

    rect_a = Rectangle((0, start_a), base_a, height_a, fill=True, color=color_a, alpha=0.3)
    ax.add_patch(rect_a)
    rect_b = Rectangle((start_b, 0), base_b, height_b, fill=True, color=color_b, alpha=0.3)
    ax.add_patch(rect_b)
    rect_final = Rectangle((start_b, 0), base_result, height_b, fill=True, color=color3, alpha=0.3)
    ax.add_patch(rect_final)
    rect_inter = Rectangle((start_b, start_a), base_c, height_c, fill=True, color=color4, alpha=0.3)
    ax.add_patch(rect_inter)
    plt.savefig(fig_name)

    # calculate the limits

    # plt.legend((color1, color2, color3, color4),
    #            ('ontology A', 'Ontology B', 'How grow Ontology A', 'Equivalents'))
    plt.text(0, start_a + height_a, label1, horizontalalignment='left', verticalalignment='bottom')
    plt.text(start_b + base_b, height_b, label2, horizontalalignment='right', verticalalignment='bottom')

    plt.text(start_b, start_a + height_a, 'Synonyms', verticalalignment='bottom')
    plt.text(start_b, height_b, 'Resultant Ontology grow', verticalalignment='bottom')

    # plt.savefig('test')

    plt.axis('off')

    plt.savefig(fig_name)

    plt.close()

    return




# import os.path


def control_inputs_numerical(variable, limitup, limitdown, variable_name):
    if isinstance(variable, float):
        if variable > limitup:
            raise ValueError(variable_name + 'exceed the up limit of' + str(limitup))
        if variable < limitdown:
            raise ValueError(variable_name + 'exceed the down limit of' + str(limitup))
    elif isinstance(variable, int):
        if variable > limitup:
            raise ValueError(variable_name + 'exceed the up limit of' + str(limitup))
        if variable < limitdown:
            raise ValueError(variable_name + 'exceed the down limit of' + str(limitup))
    else:
        raise ValueError(variable_name + 'must be numerical')
    return


def checktuples(check_current):
    current = check_current[1].split(',')
    if current.__len__() == 1:
        result_check = [eval(current[0])]
    else:
        result_check = list(map(lambda a: eval(a), current))
    tuple_check = tuple(result_check)
    return tuple_check


def establish_topological(topological):
    topological = eval(topological)
    if topological == 'blondel' or topological == 'Blondel':
        topological_test = True
        topological_cosine = False
        topological_pearson = False
        topological_euclidean = False
    elif topological == 'constriction' or topological == 'Constriction':
        topological_test = False
        topological_cosine = False
        topological_pearson = False
        topological_euclidean = False
    elif topological == 'cosine' or topological == 'Cosine':
        topological_test = True
        topological_cosine = True
        topological_pearson = False
        topological_euclidean = False
    elif topological == 'pearson' or topological == 'Pearson':
        topological_test = True
        topological_cosine = False
        topological_pearson = True
        topological_euclidean = False
    elif topological == 'euclidean' or topological == 'Euclidean':
        topological_test = True
        topological_cosine = False
        topological_pearson = False
        topological_euclidean = True
    else:
        raise ValueError("topological_similarity must be: 'blondel','constriction', 'cosine', 'pearson' or 'euclidean'")
    return topological_test, topological_cosine, topological_pearson, topological_euclidean


def parse_and_check(current_value, line, variable_str, data_type, data_type_str, special_parse=None, up_lim=None,
                    down_lim=None):
    if special_parse is None:
        if line[0][1:] == variable_str and line[1] != '\n':
            new_variable = eval(line[1])
            if not isinstance(new_variable, data_type) and new_variable is not None:
                raise ValueError(variable_str + ' must be ' + data_type_str)
            return new_variable
        else:
            return current_value
    else:
        if special_parse == 'tuple':
            if line[0][1:] == variable_str and line[1] != '\n':
                new_variable = checktuples(line)
                return new_variable
            else:
                return current_value
        elif special_parse == 'numerical':
            if line[0][1:] == variable_str and line[1] != '\n':
                new_variable = eval(line[1])
                control_inputs_numerical(new_variable, up_lim, down_lim, variable_str)
                return new_variable
            else:
                return current_value
        else:
            return current_value


def read_conf(path_to_config):
    # set default values:
    input_folder = ''
    output_folder = ''
    parallelization = False
    proc = 1
    # # Ontology_parse
    parse_ontology1 = True
    parse_ontology2 = True
    # # # Ontology 1
    file1 = None
    file_clean_ontology1 = ''
    take_synonyms1 = True
    filter_by_keywords1 = None
    ontologyname1 = None
    synonym_type1 = tuple(['owlNCBITaxon:synonym'])
    label_type1 = tuple(['rdfs:label'])
    relative_type1 = tuple(['owl:someValuesFrom'])

    ontology1_file = ''
    ontology1_name = ''

    # # # Ontology 2
    file2 = None
    file_clean_ontology2 = ''
    take_synonyms2 = True
    filter_by_keywords2 = None
    ontologyname2 = None
    synonym_type2 = tuple(['owlNCBITaxon:synonym'])
    label_type2 = tuple(['rdfs:label'])
    relative_type2 = tuple(['owl:someValuesFrom'])

    ontology2_file = ''
    ontology2_name = ''

    # # ontology_fusion
    onto1_fuseclasses = True
    onto2_fuseclasses = True
    onto1_restriction = False
    onto2_restriction = False
    onto1_list_clear = None
    onto2_list_clear = None
    synonyms = False
    text_process = False
    split_from1 = None
    split_since1 = None
    split_from2 = None
    split_since2 = None
    windowsize = 4
    globalthreshold = 0.85
    localthreshold = 0.7
    topological_test = True
    constriction_threshold = 0.0
    topological_cosine = False
    topological_pearson = False
    topological_euclidean = False
    topological_threshold = 0.0

    # read the txt

    with open(path_to_config) as f:
        lines = f.readlines()
        f.close()
    for i in range(len(lines)):
        if lines[i][0] == '>':
            current = lines[i].split('=', 1)
            # set default values:
            input_folder = parse_and_check(input_folder, current, 'input_folder', str, 'string')
            output_folder = parse_and_check(output_folder, current, 'output_folder', str, 'string')
            parallelization = parse_and_check(parallelization, current, 'parallelization', bool, 'boolean')
            proc = parse_and_check(proc, current, 'proc', int, 'integer')

            # # Ontology_parse
            parse_ontology1 = parse_and_check(parse_ontology1, current, 'parse_ontology1', bool, 'boolean')
            parse_ontology2 = parse_and_check(parse_ontology2, current, 'parse_ontology2', bool, 'boolean')

            # # # Ontology 1
            if parse_ontology1:
                file1 = parse_and_check(file1, current, 'file1', str, 'string')
                file_clean_ontology1 = parse_and_check(file_clean_ontology1, current, 'file_clean_ontology1', str,
                                                       'string')
                take_synonyms1 = parse_and_check(take_synonyms1, current, 'take_synonyms1', bool, 'boolean')
                filter_by_keywords1 = parse_and_check(filter_by_keywords1, current, 'filter_by_keywords1', '', '',
                                                      special_parse='tuple')
                ontologyname1 = parse_and_check(ontologyname1, current, 'ontologyname1', str, 'string')
                synonym_type1 = parse_and_check(synonym_type1, current, 'synonym_type1', '', '', special_parse='tuple')
                label_type1 = parse_and_check(label_type1, current, 'label_type1', '', '', special_parse='tuple')
                relative_type1 = parse_and_check(relative_type1, current, 'relative_type1', '', '',
                                                 special_parse='tuple')
            elif not parse_ontology1:
                ontology1_file = parse_and_check(ontology1_file, current, 'ontology1_file', str, 'string')
                ontology1_name = parse_and_check(ontology1_name, current, 'ontology1_name', str, 'string')

            # # # Ontology 2
            if parse_ontology2:
                file2 = parse_and_check(file2, current, 'file2', str, 'string')
                file_clean_ontology2 = parse_and_check(file_clean_ontology2, current, 'file_clean_ontology2',
                                                       str, 'string')
                take_synonyms2 = parse_and_check(take_synonyms2, current, 'take_synonyms2', bool, 'boolean')
                filter_by_keywords2 = parse_and_check(filter_by_keywords2, current, 'filter_by_keywords2', '', '',
                                                      special_parse='tuple')
                ontologyname2 = parse_and_check(ontologyname2, current, 'ontologyname2', str, 'string')
                synonym_type2 = parse_and_check(synonym_type2, current, 'synonym_type2', '', '', special_parse='tuple')
                label_type2 = parse_and_check(label_type2, current, 'label_type2', '', '', special_parse='tuple')
                relative_type2 = parse_and_check(relative_type2, current, 'relative_type2', '', '',
                                                 special_parse='tuple')
            elif not parse_ontology2:
                ontology2_file = parse_and_check(ontology2_file, current, 'ontology2_file', str, 'string')
                ontology2_name = parse_and_check(ontology2_name, current, 'ontology2_name', str, 'string')

            # # ontology_fusion
            onto1_fuseclasses = parse_and_check(onto1_fuseclasses, current, 'onto1_fuseclasses', bool, 'boolean')
            onto2_fuseclasses = parse_and_check(onto2_fuseclasses, current, 'onto2_fuseclasses', bool, 'boolean')
            onto1_restriction = parse_and_check(onto1_restriction, current, 'onto1_restriction', bool, 'boolean')
            onto2_restriction = parse_and_check(onto2_restriction, current, 'onto2_restriction', bool, 'boolean')
            onto1_list_clear = parse_and_check(onto1_list_clear, current, 'onto1_list_clear', '', '',
                                               special_parse='tuple')
            onto2_list_clear = parse_and_check(onto2_list_clear, current, 'onto2_list_clear', '', '',
                                               special_parse='tuple')
            synonyms = parse_and_check(synonyms, current, 'synonyms', bool, 'boolean')
            text_process = parse_and_check(text_process, current, 'text_process', bool, 'boolean')
            split_from1 = parse_and_check(split_from1, current, 'split_from1', str, 'string')
            split_since1 = parse_and_check(split_since1, current, 'split_since1', str, 'string')
            split_from2 = parse_and_check(split_from2, current, 'split_from2', str, 'string')
            split_since2 = parse_and_check(split_since2, current, 'split_since2', str, 'string')
            windowsize = parse_and_check(windowsize, current, 'windowsize', '', '', special_parse='numerical',
                                         up_lim=9, down_lim=1)
            globalthreshold = parse_and_check(globalthreshold, current, 'globalthreshold', '', '',
                                              special_parse='numerical', up_lim=1.0, down_lim=0.0)
            localthreshold = parse_and_check(localthreshold, current, 'localthreshold', '', '',
                                             special_parse='numerical', up_lim=1.0, down_lim=0.0)
            if current[0][1:] == 'topological_similarity' and current[1] != '\n':
                topological_test, topological_cosine, topological_pearson, topological_euclidean = \
                    establish_topological(current[1])
            topological_threshold = parse_and_check(topological_threshold, current, 'topological_threshold', '', '',
                                                    special_parse='numerical', up_lim=1.0, down_lim=0.0)

    if ontologyname1 == '':
        ontologyname1 = None
    if ontologyname2 == '':
        ontologyname2 = None
    if file_clean_ontology1 == '':
        file_clean_ontology1 = None
    if file_clean_ontology2 == '':
        file_clean_ontology2 = None
    if file1 is not None and input_folder is not None and input_folder != '':
        file1 = input_folder + file1
    if file2 is not None and input_folder is not None and input_folder != '':
        file2 = input_folder + file2
    return input_folder, output_folder, parallelization, proc, parse_ontology1, parse_ontology2, file1,\
           file_clean_ontology1,take_synonyms1, filter_by_keywords1, ontologyname1, synonym_type1, label_type1,\
           relative_type1, ontology1_file, ontology1_name, file2, file_clean_ontology2, take_synonyms2,\
           filter_by_keywords2, ontologyname2, synonym_type2, label_type2, relative_type2, ontology2_file,\
           ontology2_name, onto1_fuseclasses, onto2_fuseclasses, onto1_restriction, onto2_restriction, onto1_list_clear,\
           onto2_list_clear, synonyms, text_process, split_from1, split_since1, split_from2, split_since2, windowsize, \
           globalthreshold, localthreshold, topological_test, constriction_threshold, topological_cosine, \
           topological_pearson, topological_euclidean, topological_threshold



import math
import numpy as np
import pandas as pd
from plotly.offline import plot
import plotly.graph_objs as go
from . import doc_managing as doc


def from_dend_to_pol(dend_points_x, dend_points_y):
    '''
    Given a set of X and Y dendrogram coordinates (dcoord and icoord elements from scipy
    dendrogram, or any related set of points) transforms these points' XY cartesian coordinates
    into (r, theta) polar coordinates, used for visualization.

    :param dend_points_x: nested list with X points.
    :param dend_points_y: nested list with Y points.
    '''
   
    # First, the nested list is flattened into a usual list
    x_flatten, y_flatten = np.array(dend_points_x).flatten(), np.array(dend_points_y).flatten()

    # print(x_flatten)
    # Extreme coordinates are obtained to calculate the span of r and theta coordinates
    min_y, max_y = np.min(y_flatten), np.max(y_flatten)
    min_x, max_x = np.min(x_flatten), np.max(x_flatten)

    # print(min_y, max_y, min_x, max_x)
    # to distribute the indexes (positions) in the last row, we find the x positions whose y
    # position is the smallest

    k = 1
    for i in set(y_flatten):
        # print(k, len(x_flatten[np.argwhere(y_flatten == i)]))
        k += 1

    n_leaves = max([len(x_flatten[np.argwhere(y_flatten == i)]) for i in set(y_flatten)])

    # Now, maximum and minimum r and thetas are calculated.
    r_f = 1.0 * max_y
    r_0 = min_y
    theta_0 = 0
    theta_f = 2*math.pi*(n_leaves - 1)/(n_leaves)
    phase = np.pi/4 + 0.35

    rcoords, thetacoords = [], []

    for i in range(len(dend_points_x)):
        rcoords.append([(r_f - r) / (r_f - r_0) for r in dend_points_y[i]])
        thetacoords.append(
            [(theta_f - theta_0) * (theta - min_x) / (max_x - min_x) + phase
             for theta in dend_points_x[i]])


    return rcoords, thetacoords


def plot_html(savefile, rcoords_init, rcoords_end,
              thetacoords_init, thetacoords_end,
              colors_init, colors_end, names_init, names_end):
    data = []

    # HTML Plotting creates a dendrogram with as many colors as clusters have been created,
    # and, due to problems rendering the text (there is no option to correctly rotate
    # the text in plotly Scatter), scatter points with hover info are created.

    # First, dendrogram scatter is created by plotting lines of the dendrograms
    plotted_coords = []

    # This plots a circle to show the levels of the dendrogram
    set_r = set(np.array(rcoords_init + rcoords_end).flatten())
    for r in set_r:
        data += [go.Scatterpolar(
            r=100*[r],
            theta=np.linspace(0, 2*np.pi, 100),
            mode='lines',
            thetaunit="radians",
            hoverinfo='none',
            line=dict(
                color='pink'),
            showlegend=False,
            opacity=0.45


        )]

    
    def return_opacity(rcoords_init, thetacoords_init, rcoords_end, thetacoords_end, i):
        minx = np.min([((rcoords_init[k]*np.cos(thetacoords_init[k])-
                        rcoords_end[k]*np.cos(thetacoords_end[k])) ** 2+
                       (rcoords_init[k] * np.sin(thetacoords_init[k]) -
                        rcoords_end[k] * np.sin(thetacoords_end[k])) ** 2)**0.5 for k in range(len(rcoords_init))])
        maxx =  np.max([((rcoords_init[k]*np.cos(thetacoords_init[k])-
                        rcoords_end[k]*np.cos(thetacoords_end[k])) ** 2+
                       (rcoords_init[k] * np.sin(thetacoords_init[k]) -
                        rcoords_end[k] * np.sin(thetacoords_end[k])) ** 2)**0.5 for k in range(len(rcoords_init))])

        alpha = ((((rcoords_init[i]*np.cos(thetacoords_init[i])-
                        rcoords_end[i]*np.cos(thetacoords_end[i])) ** 2+
                       (rcoords_init[i] * np.sin(thetacoords_init[i]) -
                        rcoords_end[i] * np.sin(thetacoords_end[i])) ** 2)**0.5 - minx)/(maxx-minx))

        return 0.75*(1 - alpha) + 0.25


    for i in range(len(rcoords_init)):
        data += [go.Scatterpolar(
            r=[rcoords_init[i], rcoords_end[i]],
            theta=[thetacoords_init[i], thetacoords_end[i]],
            mode='lines',
            thetaunit="radians",
            hoverinfo='none',
            line=dict(
                color='gray'),
            showlegend=False,
            opacity= return_opacity(rcoords_init, thetacoords_init, rcoords_end, thetacoords_end, i)

        )]

    for i in range(len(rcoords_init)):
        if [rcoords_init[i],thetacoords_init[i]] not in plotted_coords:
            data += [go.Scatterpolar(
                            r = [rcoords_init[i]],
                            theta = [thetacoords_init[i]],
                            mode = 'markers',
                            thetaunit="radians",
                            hoverinfo='text',
                            hovertext=names_init[i],
                            marker =  dict(
                            color=colors_init[i],
                            size=8),
                            showlegend=False,

                        )
                    ]
            plotted_coords.append([rcoords_init[i],thetacoords_init[i]])

        if [rcoords_end[i],thetacoords_end[i]] not in plotted_coords:
            data += [go.Scatterpolar(
                            r = [rcoords_end[i]],
                            theta = [thetacoords_end[i]],
                            mode = 'markers',
                            thetaunit="radians",
                            hoverinfo='text',
                            hovertext=names_end[i],
                            marker =  dict(
                            color=colors_end[i],
                            size=8),
                            showlegend=False,

                        )
                    ]
            plotted_coords.append([rcoords_end[i],thetacoords_end[i]])







    # if labels:
    #     # Next, points were the leaves end are also added. In this case rcoords is not used
    #     # because it contains intermediate leaves. In order to get the final positions, we
    #     # calculate the angles beforehand and store it into thetatextcoords.
    #
    #     r = 1.02
    #
    #     # Names stores into the legend each point. In order not to get any repetition, we plot with
    #     # the name parameter (legend) only once: the first time they appear.
    #     saidnames = []
    #     for i in range(len(group_numbers)):
    #         if group_names[group_numbers[i]] not in saidnames:
    #             saidnames.append(group_names[group_numbers[i]])
    #             showlegend=True
    #         else:
    #             showlegend=False
    #
    #         data += [go.Scatterpolar(
    #                 r=[r],
    #                 theta=[thetatextcoords[i]],
    #                 mode='markers',
    #                 name=group_names[group_numbers[i]],
    #                 hovertext=['%s / %s' %(ticklabels[i], group_names[group_numbers[i]])],
    #                 thetaunit="radians",
    #                 marker=dict(
    #                     color=group_colors[group_numbers[i]],
    #                     size=8
    #                 ),
    #                 showlegend=showlegend)]

    # And now we remove the info of axis ticks, ticklabels, etc.
    layout = go.Layout(polar = dict(
                        domain=dict(
                            y=[0, 1],),
                          radialaxis = dict(
                            showticklabels=False,
                              showgrid=False,
                              linewidth=0,
                              ticks='',
                              tickwidth=0,
                              categoryarray=[],
                          ),
                          angularaxis = dict(
                              showticklabels=False,
                              showgrid=False,
                              linewidth=0,
                              ticks='',
                              tickwidth=0,
                              categoryarray=[]
                          )
                        ),)

    fig = go.Figure(data=data, layout=layout)

    if savefile[-5:] != '.html':
        savefile += '.html'
    # print(savefile)
    plot(fig, filename=savefile, auto_open=False)


def circular_graph(file, name):
    '''
    :param file: localization of .ods file in the specific format for taking the current coordinates
    format (for every row): [(father_num_code, son_num_code), (father node coordinate), (son node coordinate), color_father, color_son, father_label, son_label
    '''
    array = doc.import_ods(file)
    # print(array)

    graph_init_num_list, graph_end_num_list = [], []
    graph_init_x_list, graph_init_y_list = [], []
    graph_end_x_list, graph_end_y_list = [], []
    color_init_list, color_end_list = [], []
    name_init_list, name_end_list = [], []

    color_translator = {'y': 'orange', 'b': 'dodgerblue', 'r': 'darkred', 'g': 'limegreen',
                        'tab:purple': 'purple', 'palegreen2': 'palegreen', '': 'black'}

    delm = ','

    for i in array:
        graph_init_num_list.append(float(i[0][1:-1].split(delm)[0]))
        graph_end_num_list.append(float(i[0][1:-1].split(delm)[1]))
        graph_init_x_list.append(float(i[1][1:-1].split(delm)[0]))
        graph_init_y_list.append(float(i[1][1:-1].split(delm)[1]))
        graph_end_x_list.append(float(i[2][1:-1].split(delm)[0]))
        graph_end_y_list.append(float(i[2][1:-1].split(delm)[1]))
        if i[3] in color_translator.keys():
            color_init_list.append(color_translator[i[3]])
        else:
            color_init_list.append(i[3])
        if i[4] in color_translator.keys():
            color_end_list.append(color_translator[i[4]])
        else:
            color_end_list.append(i[4])
        name_init_list.append(i[5])
        name_end_list.append(i[6])

    df_graph = pd.DataFrame({'Graph init num': graph_init_num_list, 'Graph init x': graph_init_x_list,
                             'Graph init y': graph_init_y_list, 'Graph init color': color_init_list,
                             'Graph init name': name_init_list,
                             'Graph end num': graph_end_num_list, 'Graph end x': graph_end_x_list,
                             'Graph end y': graph_end_y_list, 'Graph end color': color_end_list,
                             'Graph end name': name_end_list,
                             })

    df_graph = df_graph[['Graph init num', 'Graph init x', 'Graph init y', 'Graph init color',
                         'Graph init name',
                         'Graph end num', 'Graph end x', 'Graph end y', 'Graph end color',
                         'Graph end name']]
    # print(df_graph)

    rcoords, thetacoords = from_dend_to_pol(
        dend_points_x=[graph_init_x_list, graph_end_x_list],
        dend_points_y=[graph_init_y_list, graph_end_y_list])

    # print(rcoords, '\n', thetacoords)

    plot_html(savefile=name, rcoords_init=rcoords[0], rcoords_end=rcoords[1],
              thetacoords_init=thetacoords[0], thetacoords_end=thetacoords[1],
              colors_init=color_init_list, colors_end=color_end_list,
              names_init=name_init_list, names_end=name_end_list)
    return


########### Libraries ###########
from . import ontology_fusion as of
from . import Ontology_parse as op
import os
from . import read_configuration as rc
from . import doc_managing as doc

# ########## Functions ########## #

def fontcell(path_to_config):
    '''
    # For set some argument by default add a '#' at line start
    :param: FOntCell_folder=''

    # Parallelization
    :param: parallelization=False
    :param: proc=1

    # # Ontology_parse
    :param: parse_ontology1=False
    :param: parse_ontology2=False

    # # # Ontology 1 # Only if parse_ontology1 == True
    :param: file1='///home/ra/Documentos/JavierCabau/Ontologies/CELDA_import.owl'
    :param: take_synonyms1=True
    :param: filter_by_keywords1='human', 'mouse'
    :param: ontologyName1=None
    :param: synonym_type1='owlNCBITaxon:synonym'
    :param: label_type1='CELDA:commonName xml:lang="en"', 'CELDA:commonName', 'rdfs:label'
    :param: relative_type1='owl:someValuesFrom'
    :param: edit_graph1=True
    # # # Ontology1 parsed path # only if parse_ontology1 == False
    :param: ontology1_path='parse_Celda.ods'
    :param: ontology1_name='CELDA'

    # # # Ontology 2 # Only if parse_ontology2 == True
    :param: file2=None
    :param: take_synonyms2=True
    :param: filter_by_keywords2='human', 'mouse'
    :param: ontologyName2=None
    :param: synonym_type2='owlNCBITaxon:synonym'
    :param: label_type2='rdfs:label')
    :param: relative_type2='owl:someValuesFrom'
    :param: edit_graph2=True
    # # # Ontology2 parsed path # only if parse_ontology2 == False
    :param: ontology2_path='Lifemap_syn.ods'
    :param: ontology2_name='Lifemap'

    # # ontology_fusion
    :param: onto1_fuseclasses=True
    :param: onto2_fuseclasses=True
    :param: onto1_restriction=False
    :param: onto2_restriction=True
    :param: onto1_list_clear='cell', 'cells'
    :param: onto2_list_clear='cell', 'cells'
    :param: synonyms=True
    :param: text_process=True
    :param: split_from1=None
    :param: split_since1=None
    :param: split_from2=None
    :param: split_since2=None
    :param: windowsize=4
    :param: globalthreshold=0.85
    :param: localthreshold=0.7
    :param: save_internals=True
    :param: OBO_format_result=False
    :param: save_internals_equiv=True
    :param: folder=''
    :param: draw_circular=True
    :param: constriction_threshold=0.0
    :param: topological_similarity='blondel'
    '''

    # read configurations
    root_dir = os.path.dirname(os.path.realpath(__file__))
    project_path = root_dir + os.sep + 'fontcell_files'
    project_demo_path = root_dir + os.sep + 'fontcell_demo'

    input_folder, output_folder, parallelization, proc, parse_ontology1, parse_ontology2, file1, \
    file_clean_ontology1, take_synonyms1, filter_by_keywords1, ontologyname1, synonym_type1, label_type1, \
    relative_type1, ontology1_path, ontology1_name, file2, file_clean_ontology2, take_synonyms2, \
    filter_by_keywords2, ontologyname2, synonym_type2, label_type2, relative_type2, ontology2_path, \
    ontology2_name, onto1_fuseclasses, onto2_fuseclasses, onto1_restriction, onto2_restriction, onto1_list_clear, \
    onto2_list_clear, synonyms, text_process, split_from1, split_since1, split_from2, split_since2, windowsize, \
    globalthreshold, localthreshold, topological_test, constriction_threshold, topological_cosine, topological_pearson,\
    topological_euclidean, topological_threshold = rc.read_conf(path_to_config)

    if path_to_config == project_demo_path + os.sep + 'demo_configurations.txt':
        input_folder = project_demo_path + os.sep
        output_folder = project_demo_path + os.sep + 'results' + os.sep
        file_clean_ontology1 = project_demo_path + os.sep + 'file_clean_CELDA.txt'

    # perform a ontology parse
    if parse_ontology1:
        ontology1, onto_name1 = parser(file1, take_synonyms1, filter_by_keywords1, ontologyname1, synonym_type1,
                                       label_type1, relative_type1, file_clean_ontology1, input_folder=input_folder,
                                       output_folder=output_folder)
    else:
        path1 = input_folder + ontology1_path
        ontology1 = doc.import_ods(path1)
        onto_name1 = ontology1_name

    if parse_ontology2:
        ontology2, onto_name2 = parser(file2, take_synonyms2, filter_by_keywords2, ontologyname2, synonym_type2,
                                       label_type2, relative_type2, file_clean_ontology2, input_folder=input_folder,
                                       output_folder=output_folder)
    else:
        path2 = input_folder + ontology2_path
        ontology2 = doc.import_ods(path2)
        onto_name2 = ontology2_name

    # run parallelization
    if parallelization:
        # generates a csv of both ontologies and a parallel_configurations.txt
        config_path = os.path.join(root_dir, 'fusion_parallelized.py')
        doc.save_csv(ontology1, project_path + os.sep + 'onto1_for_parallel.csv')
        doc.save_csv(ontology2, project_path + os.sep + 'onto2_for_parallel.csv')
        list_of_variables = [onto_name1, onto_name2, file1, onto1_fuseclasses, onto2_fuseclasses, onto1_restriction,
                             onto2_restriction, onto1_list_clear, onto2_list_clear, synonyms, text_process, split_from1,
                             split_since1, split_from2, split_since2, windowsize, globalthreshold, localthreshold,
                             parallelization, topological_test, constriction_threshold, topological_cosine,
                             topological_pearson, topological_euclidean, topological_threshold]
        list_of_variables_str = list(map(lambda a: str(a)+'\n', list_of_variables))
        doc.save_txt(list_of_variables_str, project_path + os.sep + 'parallelization_config.txt')

        # launch a subprocess with mpirun and the parallelization (adding proc variable)
        cmd = "mpirun -np " + str(proc) + " python3 " + config_path
        os.system(cmd)

    else:
        # perform the align and merge/fusion
        of.fuseOntologies3(ontology1, ontology2, onto_name1, onto_name2,
                        onto1_path=file1,
                        onto1_fuseclasses=onto1_fuseclasses,
                        onto2_fuseclasses=onto2_fuseclasses,
                        onto1_restriction=onto1_restriction,
                        onto2_restriction=onto2_restriction,
                        onto1_list_clear=onto1_list_clear,
                        onto2_list_clear=onto2_list_clear,
                        synonyms=synonyms,
                        text_process=text_process,
                        split_from1=split_from1,
                        split_since1=split_since1,
                        split_from2=split_from2,
                        split_since2=split_since2,
                        windowsize=windowsize,
                        globalthreshold=globalthreshold,
                        localthreshold=localthreshold,
                        constriction_threshold=constriction_threshold,
                        topological_threshold=topological_threshold,
                        parallelization=parallelization,
                        topological_test=topological_test,
                        topological_cosine=topological_cosine,
                        topological_pearson=topological_pearson,
                        topological_euclidean=topological_euclidean,
                        output_folder=output_folder)
    return


def parser(file, take_synonyms, filter_by_keywords, ontologyname, synonym_type, label_type, relative_type,
           file_for_clean=None, input_folder=None, output_folder=None):
    ontology_parsed, ontology_name = op.parse_ontology(file, take_synonyms, filter_by_keywords, ontologyname, synonym_type,
                                                    label_type, relative_type, input_folder=input_folder,
                                                    output_folder=output_folder)
    if file_for_clean is not None:
        ontology_edited = op.editGraph(ontology_parsed, input_folder + file_for_clean, output_file=output_folder,
                                    ontoName=ontology_name)
        return ontology_edited, ontology_name
    else:
        return ontology_parsed, ontology_name



#
# Document with tools for write a html using python


def html(corpus):
    return '<!DOCTYPE html>' + '<html>' + corpus + '</html>'


def tittle(tittle_string):
    return '<head>' + tittle_string + '</head>'


def body(corpus):
    return '<body>' + corpus + '</body>'


def heading(text, number=None):
    if number is None:
        return '<h>' + text + '</h>'
    else:
        return '<h' + str(number) + '>' + text + '</h' + str(number) + '>'


def paragraph(text, number=None):
    if number is None:
        return '<p>' + text + '</p>'
    else:
        return '<p' + str(number) + '>' + text + '</p' + str(number) + '>'


def htmllist(list_of_strings):
    text = list(map(lambda a: '<li>' + a + '</li>', list_of_strings))
    result = ''.join(text)
    return '<ul>' + result + '</ul>'


def insert_link(link, mask_string):
    return '<a href="' + link + '">' + mask_string + '</a>'


def insert_image(image_name, width=200, height=200, align=None):
    if align is None:
        return '<img src="' + image_name + '" width="' + str(width) + '" height="' + str(height) + '">'
    else:
        return '<img src="' + image_name + '" width="' + str(width) + '" height="' + str(height) + '"align="' \
               + align + '">'

def blankspace():
    return '<br>'


def align(corpus, side):
    return '<div align="' + side + '">' + corpus + '</div>'


def add_box(corpus, type):
    return '<div class ="' + type + '" >' + corpus + '</div>'


def add_style(list_styles, type=None, number=None, style=None):
    if list_styles == '':
        return tittle('<style>' + type + str(number) + '{' + style + '}' + '</style> ')
    else:
        old = list_styles.split('<head><style>')[1]
        new = '<head><style>' + type + str(number) + '{' + style + '}'
        return new + old



#######################################################################################################################
                                                # Import Libraries #
#######################################################################################################################
import numpy as np
from itertools import chain
from . import doc_managing as doc
import os.path

#######################################################################################################################
                                                    # Functions #
#######################################################################################################################


def splited(soup, t=False):
    x = list(chain(*list(map(lambda a:a.split('<'), soup.split('\n')))))

    y = list(chain(*list(map(lambda a:a.split('>'), x))))

    if t:
        var = list(chain(*list(map(lambda a:a.split(':'), y))))
    if not t:
        var = y

    final = list(map(lambda a:a.split(' '), var))
    return final


def splitlist(list, key):
    for i in range(len(list)):
        list[i] = list[i].split(key)
    return list


def search_ontology_name(ontology_splited):
    '''
    Find the 'name' of the ontology.
    '''
    names = []
    for i in range(ontology_splited.__len__()):
        if ontology_splited[i][0] == '!--':
            try:
                variable = ontology_splited[i][1].split('_')[0]
                name = str(variable.split('/')[-1]) + '_'
                try:
                    name = str(name.split('#')[1])
                    names.append(name)
                except:
                    names.append(name)
            except:
                continue

    repetitions = {}
    for j in range(len(names)):
        if names[j] not in repetitions:
            repetitions[names[j]] = 1
        elif names[j] in repetitions:
            value = repetitions[names[j]]
            repetitions[names[j]] = value + 1

    current_value = 0
    for key, value in repetitions.items():
        if value > current_value:
            current_value = value
            ontologyname = key

    return ontologyname


def parse_ontology(file, take_synonyms=True, filter_by_keywords=None, ontologyName=None,
                   synonym_type=('owlNCBITaxon:synonym'), label_type= ('CELDA:commonName xml:lang="en"', 'CELDA:commonName', 'rdfs:label'),
                   relative_type=('owl:someValuesFrom'), input_folder=None, output_folder=None):
    # read the ontology and generate a full splited ontology
    # for organism filter is needed the name give as argument present in the ontology labels.
    #First name at: recover[i][0][0]
    if isinstance(synonym_type, str):
        synonym_type1 = [synonym_type]
    elif isinstance(synonym_type, tuple):
        synonym_type1 = list(synonym_type)
    elif isinstance(synonym_type, list):
        synonym_type1 = synonym_type
    else:
        raise ValueError('synonym_type not valid')

    if isinstance(label_type, str):
        label_type1 = [label_type]
    elif isinstance(label_type, tuple):
        label_type1 = list(label_type)
    elif isinstance(label_type, list):
        label_type1 = label_type
    else:
        raise ValueError('label_type not valid')

    if isinstance(relative_type, str):
        relative_type1 = [relative_type]
    elif isinstance(relative_type, tuple):
        relative_type1 = list(relative_type)
    elif isinstance(relative_type, list):
        relative_type1 = relative_type
    else:
        raise ValueError('relative_type not valid')

    with open(file, 'r') as f:
        soup = f.read()

    final_soup = splited(soup)

    recover = []
    clase = False
    for i in range(len(final_soup)):
        # print(i)
        if final_soup[i][0] == 'owl:Class':
            clase = True
            if take_synonyms:
                label = []
            ancest = []
            j = 0
            #labeled = False
            # print('class find at i: ' + str(i))

        while clase:
            # print(' ', j)
            # if i == 9516 and j == 59:
            # print()
            if i + j + 1 > len(final_soup):
                break
            # print('current i + j: ' + str(i+j))
            if final_soup[i + j][0] in relative_type1 and len(final_soup[i + j]) > 1:
                if str(final_soup[i + j][1]) != '':
                    ancest.append(final_soup[i + j][1])

            if final_soup[i + j][0] in label_type1: #and not labeled:
                if str(final_soup[i + j + 1]) != '':
                    if take_synonyms:
                        label.append(final_soup[i + j + 1])
                    else:
                        label = final_soup[i + j + 1]
                    #labeled = True
            if final_soup[i + j][0] in synonym_type1 and take_synonyms:
                if str(final_soup[i + j + 1]) != '':
                    label.append(final_soup[i + j + 1])

            if final_soup[i + j][0] == '/owl:Class':
                if label != '':
                    recover.append([label, ancest])
                    j = 0
                    label = ''
                    clase = False
                else:
                    j = 0
                    clase = False

            else:
                j = j + 1

    for i in range(len(recover)):
        for j in range(len(recover[i][0])):
            recover[i][0][j] = ' '.join(recover[i][0][j])
        try:
            # Remove 'rdf:resource='
            ascn = splitlist(recover[i][1], 'rdf:resource=')
            #ascn1 = list(map(lambda a:a.split('rdf:resource='), recover[i]))
            aux_list = []
            for j in range(len(ascn)):
                aux_list.append(ascn[j][1])
            ascn = aux_list
            # recover[i][1] = ascn

            # Remove '"'
            aux_list = []
            ascn = splitlist(ascn, '"')
            #ascn2 = list(map(lambda a:a.split('"'), ascn1))
            for k in range(len(ascn)):
                aux_list.append(ascn[k][1])
            ascn = aux_list
            # recover[i][1] = ascn

            # Remove '#' if there're
            for n in range(len(ascn)):
                try:
                    ascn[n] = ascn[n].split('#')[1]
                except:
                    ascn[n] = ascn[n]
            recover[i][1] = ascn

        except:
            recover[i][1] = recover[i][1]

        # print(recover[i])
    # print(len(recover))

    # Post processing: from list of list to a single sort list.
    final = np.zeros(len(recover), dtype=object)

    for i in range(len(recover)):
        auxiliar = [recover[i][0]]
        for k in range(len(recover[i][1])):
            auxiliar.append(recover[i][1][k])
        final[i] = auxiliar

    # print('Ontology parsed')
    if ontologyName == None:
        name = search_ontology_name(final_soup)
    else:
        name = ontologyName

    final = clearing_ascendants(final, name)
    # Check if a dictionary of IDs_label exist
    # return final
    # r_path = os.path.dirname(os.path.realpath(__file__))
    path_to_doc = output_folder
    # print(path_to_doc)
    if not os.path.exists(path_to_doc + name + 'Dictionary_IDS.ods'):
        print('Making ID\'s dictionary')
        IDs1 = makedictionary_ofIDs_onlyCurrentOntology(final_soup, name)
        doc.save_ods(IDs1, path_to_doc + name + 'Dictionary_IDS.ods')
        IDs = doc.import_ods(path_to_doc + name + 'Dictionary_IDS.ods', 'pyexcel sheet')
        IDs = parse_import_odsDict(IDs)
    else:
        try:
            print('Importing ID\'s dictionary')
            IDs = doc.import_ods(path_to_doc + name + 'Dictionary_IDS.ods', 'pyexcel sheet')
            IDs = parse_import_odsDict(IDs)
        except:
            print('Making ID\'s dictionary')
            IDs = makedictionary_ofIDs_onlyCurrentOntology(final_soup, name)
            doc.save_ods(IDs, path_to_doc + name + 'Dictionary_IDS.ods')

    # print('Setting ascendants labels')
    ontology = ID_to_label(final, IDs)
    # return ontology
    # print('Setting ascendant-descendant relation.')
    ontology_sorted = set_parent_son_relations(ontology)

    # set all .lower()
    for i in range(len(ontology_sorted)):
        if len(ontology_sorted[i]) == 1:
            ontology_sorted[i][0][0] = list(map(lambda a: a.lower(), ontology_sorted[i][0][0]))
        elif len(ontology_sorted[i]) == 2:
            ontology_sorted[i][0][0] = list(map(lambda a: a.lower(), ontology_sorted[i][0][0]))
            ontology_sorted[i][1][0] = list(map(lambda a: a.lower(), ontology_sorted[i][1][0]))
    # return ontology_sorted

    # return ontology_sorted
    if filter_by_keywords is not None:
        filter_by_keywords1 = list(filter_by_keywords)
        ontology_sorted = filter_bykeywords(ontology_sorted, filter_by_keywords1)
    # if filter_humanMouse:
    #      ontology_sorted = filter_human_mouse(ontology_sorted)
    # # return ontology_sorted
    return ontology_sorted, name


def parse_import_odsDict(IDs):
    '''
    Parse the definitions format to set a list format instead of a 'str' format.
    '''
    new_IDs = np.zeros((len(IDs), 2), dtype=object)
    for i in range(len(new_IDs)):
        new_IDs[i][0] = IDs[i][0]
        # parse and delete '[' and ']'
        new_IDs[i][1] = IDs[i][1].split('\'')[1:-1]
        # delete ','
        new_IDs[i][1] = list(filter(lambda a: a != ', ', new_IDs[i][1]))
    return new_IDs


def makedictionary_ofIDs_onlyCurrentOntology(splited_ontology, ontology_name):
    # Only takes names for our current ontology at the IDs dictionary
    clase = False
    dictionary = []
    for i in range(len(splited_ontology)):
        if splited_ontology[i][0] == 'owl:Class':
            clase = True
            label = []
            ID = ''
            # j = 0
            # print('class find at i: ' + str(i))

        while clase:
            try:
                ID = ontology_name + splited_ontology[i][1].split(ontology_name)[1]
                ID = ID.split('"')[0]
                ID_onto = True
            except:
                clase = False
                ID_onto = False

            if ID_onto:
                fully_labeled = False
                j = 0
                while not fully_labeled:
                    if splited_ontology[i + j][0] == 'rdfs:label':
                        if str(splited_ontology[i + j + 1]) != '':
                            label.append(splited_ontology[i + j + 1])
                    if splited_ontology[i + j][0] == 'CELDA:commonName xml:lang="en"' or splited_ontology[i + j][0] == 'CELDA:commonName':
                        if str(splited_ontology[i + j + 1]) != '':
                            label.append(splited_ontology[i + j + 1])
                    if splited_ontology[i + j][0] == 'owlNCBITaxon:synonym':
                        if str(splited_ontology[i + j + 1]) != '':
                            label.append(splited_ontology[i + j + 1])
                    if splited_ontology[i + j][0] == '/owl:Class':
                        clase = False
                        ID_onto = False
                        fully_labeled = True
                        dictionary.append([ID, label])
                    j += 1

    for i in range(len(dictionary)):
        for j in range(len(dictionary[i][1])):
            aux = ''
            for k in range(len(dictionary[i][1][j])):
                aux = aux + ' ' + dictionary[i][1][j][k]
            dictionary[i][1][j] = aux.strip()

    return dictionary


def ID_to_label(onto_raw, dictionary):
    final = np.zeros(len(onto_raw), dtype=object)
    a = dictionary[:,0]
    b = dictionary[:,1]
    celda_Ids = list(a)
    id_names = list(b)

    for i in range(len(onto_raw)):
        current =  []
        for j in range(len(onto_raw[i])):
            # Add ID
            if onto_raw[i][j] in id_names:
                current.append([onto_raw[i][0], celda_Ids[id_names.index(onto_raw[i][j])]])
            # Add name
            if onto_raw[i][j] in celda_Ids:
                current.append([id_names[celda_Ids.index(onto_raw[i][j])], onto_raw[i][j]])
        final[i]=current
    return final


def clearing_ascendants(raw_ontology, name):
    for i in range(len(raw_ontology)):
        for j in range(len(raw_ontology[i])):
            if j > 0:
                try:
                    raw_ontology[i][j] = name + raw_ontology[i][j].split(name)[1]
                except:
                    continue
    return raw_ontology


def set_parent_son_relations(ascendant_relations):
    ascendant_relations1 = ascendant_relations.tolist()
    new_relation = []

    for i in range(len(ascendant_relations1)):
        if len(ascendant_relations1[i]) == 1:
            pass
        elif len(ascendant_relations1[i]) == 2:
            ascendant_relations1[i] = [ascendant_relations1[i][1],ascendant_relations1[i][0]]
        else:
            take = False
            for j in range(len(ascendant_relations1[i])):
                if j == 1 and j != 0:
                    to_substitute = [ascendant_relations1[i][j],ascendant_relations1[i][0]]
                    take = True
                elif j != 0 and j != 1:
                    new_relation.append([ascendant_relations1[i][j], ascendant_relations1[i][0]])
            if take:
                ascendant_relations1[i] = to_substitute
    final = ascendant_relations1 + new_relation
    return final

    new_relation = []
    for i in range(len(ascendant_relations)):
        if len(ascendant_relations[i]) == 1:
            pass
        else:
            for j in range(len(ascendant_relations[i][1])):
                if not [ascendant_relations[i][1], ascendant_relations[i][0]] in new_relation:
                    new_relation.append([ascendant_relations[i][1], ascendant_relations[i][0]])
    return new_relation


def editGraph(ontology, file, output_file=None, ontoName=None):
    '''
    This tool create this 'ideal graph' for CELDA ontology (mouse or human).
    :param ontology: ontology filtered for human or mouse
    :param file: file path to a .txt with the info for modifying
        Code:
        'label'                 --> deletes the class that contains this label
        [[class1], [class2]]    --> add new relations between classes
        ID:'ID_number'          --> delete the classes with this ID
        f[[class1], [class2]]   --> Fuse two classes

    :return: an Ideal graph
    '''
    # Read the file

    for_clear, to_add, IDs_todelete, to_fuse, new_node = readFC(file)

    # Delete non clear cell types (or without relevant information about cell types, better say duplications) which
    # they are adams or they are in adam's graphs.Delete those nodes that has a very long character string.
    for i in range(len(ontology)):
        if not ontology[i]:
            continue
        if ontology[i][0][1] in IDs_todelete and ontology[i] == []:
            ontology[i] = []
            continue
        # father
        sentence = list(chain(*list(map(lambda a:a.split(), ontology[i][0][0]))))
        if 'derived' in sentence or not plausible_celltype(sentence):
            for k in range(len(ontology[i][0][0])):
                for_clear.append(ontology[i][0][0][k])
        # son
        if ontology[i].__len__() > 1:
            if ontology[i][1][1] in IDs_todelete:
                ontology[i] = []
                continue
            sentence = list(chain(*list(map(lambda a:a.split(), ontology[i][1][0]))))
            if 'derived' in sentence or not plausible_celltype(sentence):
                for k in range(len(ontology[i][1][0])):
                    for_clear.append(ontology[i][1][0][k])

    ontology_cleared = []

    for i in range(ontology.__len__()):
        if not ontology[i]:
            continue
        father = ontology[i][0][0]
        if ontology[i].__len__() > 1:
            son = ontology[i][1][0]
        else:
            son = []

        if len(set(father).intersection(for_clear)) == 0 and len(set(son).intersection(for_clear)) == 0:
            ontology_cleared.append(ontology[i])
    # add new_nodes

    for i in range(len(new_node)):
        ontology_cleared.append(new_node[i])

    # Create new relations

    full_ontology = np.zeros([(ontology_cleared.__len__() + to_add.__len__()), 2], dtype=object)

    for k in range(ontology_cleared.__len__()):
        full_ontology[k] = ontology_cleared[k]
    for n in range(to_add.__len__()):
        full_ontology[ontology_cleared.__len__()+n] = to_add[n]

    # Fuse classes:
    full_ontology = list(full_ontology)
    for i in range(len(to_fuse)):
        current = to_fuse[i][0] + to_fuse[i][1]
        for j in range(len(full_ontology)):
            if full_ontology[j][0][0] == to_fuse[i][0] or full_ontology[j][0][0] == to_fuse[i][1]:
                full_ontology[j][0][0] = current
            if full_ontology[j][1][0] == to_fuse[i][0] or full_ontology[j][1][0] == to_fuse[i][1]:
                full_ontology[j][1][0] = current
    # Clearing for re-compensate the ontology modified (deleting repetitions and autoassignations), delete loops too
    full_ontology = clear_ontology(full_ontology)
    # save_ods(full_ontology, '///home/ra/Documentos/testing_folder_output/test_celda.ods')

    # final check
    set_adams(full_ontology, output_file, ontoName)

    return full_ontology


def set_adams(ontology, outfolder, name):
    '''
    for 'Adam' we understand a root node, a node with no father. The ideal, in a biological cell type context, graph or
    simulation only has one root node.
    Whit this tool you can see whats are the adams in your graph for clear them (adding it to some branch or clear them
    if they don't have relevant information)
    '''
    adams = []
    for i in range(len(ontology)):
        sit = False
        for j in range(len(ontology)):
            descen = ontology[j][1:]
            if ontology[i][0] in descen:
                sit = True
            else:
                continue
        if not sit and ontology[i][0] not in adams:
            adams.append(ontology[i][0])
    if len(adams) > 1:
        print('More than one plausible root')
        # print(adams)
        f = open(outfolder + os.sep + name + '_plausibles_roots.txt', 'w')
        for i in range(len(adams)):
            f.write(str(adams[i]) + '\n')
        print('Data save as "plausibles_roots.txt" at output folder')
        o = open(outfolder + os.sep + name + '_ontology_with_various_roots.txt', 'w')
        for i in range(len(ontology)):
            o.write(str(ontology[i]) + '\n')
        print('Created adittional document at output folder with the ontology parsed named as: '
              '"ontology_with_various_roots.txt"')
        o.close()
        f.close()
    return


def clear_ontology(ontology):

    # Clear autoassignations
    no_rep = np.zeros((ontology.__len__(),2),dtype=object)
    for i in range(len(ontology)):
        if ontology[i].__len__()>1:
            if ontology[i][0] != ontology[i][1]:
                no_rep[i][0] = ontology[i][0]
                no_rep[i][1] = ontology[i][1]
        else:
            no_rep[i][0] = ontology[i][0]

    cleared =  []
    for i in range(len(no_rep)):
        if no_rep[i][0] != 0 and no_rep[i][1] != 0:
            row = [no_rep[i][0], no_rep[i][1]]
            cleared.append(row)
        elif no_rep[i][0] != 0 and no_rep[i][1] == 0:
            row = no_rep[i][0]
            cleared.append(row)
        else:
            continue

    # Clear double assignations
    final_cleared= []
    deleted = []
    for i in range(len(cleared)):
        repeat = False
        count = 0
        for j in range(len(cleared)):
            if cleared[i] == cleared[j] and cleared[j] not in deleted:
                if count == 1:
                    repeat = True
                    deleted.append(cleared[j])
                    break
                else:
                     count = 1
        if not repeat:
            final_cleared.append(cleared[i])

    # clear loops


    return final_cleared


def plausible_celltype(list_of_string):
    celltype = False
    for i in range(len(list_of_string)):
        try:
            test = list_of_string[i].split('cyte')
            celltype = True
            break
        except:
            celltype = False
        try:
            test = list_of_string[i].split('blast')
            celltype = True
            break
        except:
            celltype = False
    return celltype


def readFC(file_path):
    '''
    From a txt, takes the 'to delete' and the 'to add' to the final ontology graph.
    :return: to delete, to add
    '''
    file = open(file_path, 'r')
    lines = file.readlines()
    file.close()
    to_delete = []
    to_add =[]
    IDs = []
    to_fuse = []
    new_node = []
    for i in range(lines.__len__()):
        if lines[i][0] == '[':
            to_add.append(eval(lines[i]))
        elif lines[i][0] == '\'':
            to_delete.append(eval(lines[i]))
        elif lines[i][0] == 'I':
            IDs.append(eval(lines[i][3:]))
        elif lines[i][0] == 'f':
            to_fuse.append(eval(lines[i][1:]))
        elif lines[i][0] == 'n':
            new_node.append(eval(lines[i][1:]))
    return to_delete, to_add, IDs, to_fuse, new_node


def parseSubClass_onto(final_soup, name, path):
    '''
    From a ontology with owl format, take the classes and they subclasses has a edge-like structure
    :param final_soup: oa splitted and computed ontology
    :return: edge-like structure
    '''
    lpath = path.split(os.sep)[:-1]
    path=''
    for i in range(len(lpath)):
        if i > 0:
            path = path + os.sep + lpath[i]
    recover = []
    clase = False
    for i in range(len(final_soup)):
        # print(i)
        if final_soup[i][0] == 'owl:Class':
            clase = True
            label = ''
            ancest = []
            j = 0
            # labeled = False
            # print('class find at i: ' + str(i))

        while clase:
            # print(' ', j)
            # if i == 9516 and j == 59:
            # print()
            if i + j + 1 > len(final_soup):
                break
            # print('current i + j: ' + str(i+j))
            if final_soup[i + j][0] == 'rdfs:subClassOf' and len(final_soup[i + j]) > 1:
                if str(final_soup[i + j][1]) != '':
                    ancest.append(final_soup[i + j][1])

            if final_soup[i + j][0] == 'rdfs:label':  # and not labeled:
                if str(final_soup[i + j + 1]) != '':
                    label = final_soup[i + j + 1]
                    # labeled = True

            if final_soup[i + j][0] == '/owl:Class':
                if label != '':
                    recover.append([label, ancest])
                    j = 0
                    label = ''
                    clase = False
                else:
                    j = 0
                    clase = False

            else:
                j = j + 1

    for i in range(len(recover)):
        aux = ''
        for k in range(len(recover[i][0])):
            if k > 0:
                aux = aux + ' ' + recover[i][0][k]
            else:
                aux = recover[i][0][k]
        recover[i][0] = aux

        try:
            # Remove 'rdf:resource='
            ascn = splitlist(recover[i][1], 'rdf:resource=')
            aux_list = []
            for j in range(len(ascn)):
                aux_list.append(ascn[j][1])
            ascn = aux_list
            # recover[i][1] = ascn

            # Remove '"'
            aux_list = []
            ascn = splitlist(ascn, '"')
            for k in range(len(ascn)):
                aux_list.append(ascn[k][1])
            ascn = aux_list
            # recover[i][1] = ascn

            # Remove '#' if there're
            for n in range(len(ascn)):
                try:
                    ascn[n] = ascn[n].split('#')[1]
                except:
                    ascn[n] = ascn[n]
            recover[i][1] = ascn

        except:
            recover[i][1] = recover[i][1]

        # print(recover[i])
    # print(len(recover))

    # Post processing: from list of list to a single sort list.
    final = np.zeros(len(recover), dtype=object)

    for i in range(len(recover)):
        auxiliar = []
        auxiliar.append(recover[i][0])
        for k in range(len(recover[i][1])):
            auxiliar.append(recover[i][1][k])
        final[i] = auxiliar
    # print('Ontology parsed')
    # name = search_ontology_name(final_soup)

    final = clearing_ascendants(final, name)
    # Check if a dictionary of IDs_label exist

    IDs = makedictionary_ofIDs_onlyCurrentOntology(final_soup, name)
    doc.save_ods(IDs, path + name + 'Dictionary_IDS.ods')

    # print('Setting ascendants labels')

    ontology1 = ID_to_label(final, IDs)

    # print('Setting ascendant-descendant relation.')
    ontology_sorted = set_parent_son_relations(ontology1)

    return ontology_sorted


def evaluate_toadds(clearedontology, toadd):
    '''
    Tool for change the names of toadd to a clearedontology format
    '''
    all_types = np.zeros([clearedontology.__len__() * 2],dtype=object)
    number = 0
    while number+1 <= all_types.__len__():
        print(number)
        all_types[number] = clearedontology[int(number//2)][0]
        all_types[number+1] = clearedontology[int(number/2)][1]
        number += 2
    all_types = np.unique(all_types)

    for i in range(len(toadd)):
        for j in range(len(all_types)):
            if toadd[i][0] in all_types[j]:
                toadd[i][0] = all_types[j]
            if toadd[i][1] in all_types[j]:
                toadd[i][1] = all_types[j]
    return toadd


def filter_human_mouse(ontology):
    # filter human or mouse nodes
    filter_ontology = []
    for i in range(len(ontology)):
        event = False
        event1 = False
        if len(ontology[i]) >= 1:
            if ontology[i][0][0].__len__() == 1:
                words = list(chain(*list(map(lambda a: a.split(), ontology[i][0][0]))))
                if 'human' in words or 'mouse' in words:
                    ontology[i][0][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][0][0]))[0][1:])]
                    event = True
                    event1 = True
                if ontology[i].__len__() >1:
                    words = list(chain(*list(map(lambda a: a.split(), ontology[i][1][0]))))
                    if 'human' in words or 'mouse' in words:
                        ontology[i][1][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][1][0]))[0][1:])]
                        event = True
                        event1 = True
            else:
                words = list(chain(*list(map(lambda a: a.split(), ontology[i][0][0]))))
                #for j in range(len(words)):
                if 'human' in words or 'mouse' in words:
                    event = True
        if event and not event1:
            # delete 'human' or 'mouse' labeled synonyms and make .unique() to synonyms and to a 'class_list'
            synonyms = []

            if len(ontology[i]) == 1:
                for j in range(len(ontology[i][0][0])):
                    if 'human' not in ontology[i][0][0][j] and 'mouse' not in ontology[i][0][0][j]:
                        synonyms.append(ontology[i][0][0][j])
                ontology[i][0][0] = synonyms

            # if we found descendants
            if len(ontology[i]) > 1:
                synonyms= []
                for j in range(len(ontology[i][0][0])):
                    if 'human' not in ontology[i][0][0][j] and 'mouse' not in ontology[i][0][0][j]:
                        synonyms.append(ontology[i][0][0][j])
                if not synonyms:
                    ontology[i][0][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][0][0]))[0][1:])]
                else:
                    ontology[i][0][0] = synonyms
                synonyms = []
                # son
                for k in range(len(ontology[i][1][0])):
                    if 'human' not in ontology[i][1][0][k] and 'mouse' not in ontology[i][1][0][k]:
                        synonyms.append(ontology[i][1][0][k])
                if not synonyms:
                    ontology[i][1][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][1][0]))[0][1:])]
                else:
                    ontology[i][1][0] = synonyms

            # delete empty cells if there're
            if not synonyms:
                pass
            else:

                filter_ontology.append(ontology[i])
                # print(i, ontology[i])
        elif event and event1:
            filter_ontology.append(ontology[i])
            # print(i, ontology[i])
    return filter_ontology


def filter_bykeywords(ontology, words_for_filter):
    # filter human or mouse nodes
    filter_ontology = []
    words_set = set(words_for_filter)
    for i in range(len(ontology)):
        event = False
        event1 = False
        if len(ontology[i]) >= 1:
            if ontology[i][0][0].__len__() == 1:
                words = list(chain(*list(map(lambda a: a.split(), ontology[i][0][0]))))
                if words_set.intersection(words).__len__() > 0:
                    ontology[i][0][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][0][0]))[0][1:])]
                    event = True
                    event1 = True
                if ontology[i].__len__() >1:
                    words = set(chain(*list(map(lambda a: a.split(), ontology[i][1][0]))))
                    if words_set.intersection(words).__len__() > 0:
                        ontology[i][1][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][1][0]))[0][1:])]
                        event = True
                        event1 = True
            else:
                words = list(chain(*list(map(lambda a: a.split(), ontology[i][0][0]))))
                #for j in range(len(words)):
                if words_set.intersection(words).__len__() > 0:
                    event = True
        if event and not event1:
            # delete word_set labeled synonyms and make '.unique()' to synonyms and to a 'class_list'
            synonyms = []

            if len(ontology[i]) == 1:
                for j in range(len(ontology[i][0][0])):
                    if words_set.intersection(ontology[i][0][0][j].split()).__len__() == 0:
                        synonyms.append(ontology[i][0][0][j])
                ontology[i][0][0] = synonyms

            # if we found descendants
            if len(ontology[i]) > 1:
                synonyms = []
                for j in range(len(ontology[i][0][0])):
                    if words_set.intersection(ontology[i][0][0][j].split()).__len__() == 0:
                        synonyms.append(ontology[i][0][0][j])
                if not synonyms:
                    ontology[i][0][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][0][0]))[0][1:])]
                else:
                    ontology[i][0][0] = synonyms
                synonyms = []
                # son
                for k in range(len(ontology[i][1][0])):
                    if words_set.intersection(ontology[i][1][0][k].split()).__len__() == 0:
                        synonyms.append(ontology[i][1][0][k])
                if not synonyms:
                    ontology[i][1][0] = [' '.join(list(map(lambda a: a.split(), ontology[i][1][0]))[0][1:])]
                else:
                    ontology[i][1][0] = synonyms

            # delete empty nodes if there're
            if not synonyms:
                pass
            else:

                filter_ontology.append(ontology[i])
                # print(i, ontology[i])
        elif event and event1:
            filter_ontology.append(ontology[i])
            # print(i, ontology[i])
    return filter_ontology



import setuptools

with open("README.md", 'r') as f:
     long_description = f.read()

setuptools.setup(
   name='fontcell',
   version='1.0.9.1',
   description='FOntCell - A software tool for automatic parallel computed fusion of ontologies',
   long_description=long_description,
   license="LICENSE",
   author='Javier Cabau Laporta',
   author_email='jcabaulaporta@outlook.es',
   url="https://gitlab.com/jcabaulaporta/FOntcell",
   install_requires=['StringDist>=1.0.9', 'graphviz>=0.10.1', 'matplotlib>=2.2.4', 'networkx>=2.2', 'numpy>=1.16.1',
                     'pandas>=0.24.1', 'plotly>=3.6.1', 'pyexcel>=0.5.12', 'pyexcel-ods>=0.5.4', 'seaborn>=0.9.0',
                     'bigmpi4py>=1.2.3', 'mpi4py>=3.0.1'],
   include_package_data=True,
   packages=setuptools.find_packages(),
   data_files=[('FOntCell/fontcell_demo',['FOntCell/fontcell_demo/logo.png']),
               ('FOntCell/fontcell_demo', ['FOntCell/fontcell_demo/CELDA_import.owl']),
               ('FOntCell/fontcell_demo', ['FOntCell/fontcell_demo/Lifemap_parsed.ods']),
               ('FOntCell/fontcell_demo', ['FOntCell/fontcell_demo/file_clean_CELDA.txt']),
               ('FOntCell/fontcell_demo', ['FOntCell/fontcell_demo/demo_configurations.txt'])],
   classifiers=[
      "Programming Language :: Python :: 3",
      "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
      "Operating System :: OS Independent",
   ],
)
